/****
 *
 * Exemplo: Extrao de Bits Mais Significativos (MSBs)
 *
 * Seo: B.5.2
 *
 * Descrio: Obtm os bits mais significativos de um nmero
 *            inteiro
 *
 ****/

#include <limits.h>
#include <stdio.h>
#include <stdlib.h>

   /* Macro utilizada em tratamento */
   /* de condies de exceo       */
#define ASSEGURA(condicao, msg) if (!(condicao)) {\
                                   fprintf(stderr, "\n%s\n", msg);\
                                   exit(1); \
                                }

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeInteiro(): L um nmero inteiro via teclado
 *              e deixa o buffer intacto
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero inteiro lido
 *
 ****/
int LeInteiro(void)
{
   int num, /* O nmero lido */
       teste, /* Valor retornado por scanf() */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Desvia para c se o valor for invlido */
inicio:
      /* Tenta ler um valor vlido */
   teste = scanf("%d", &num);

      /* Se no ocorreu erro de leitura ou de final */
      /* de arquivo, h caracteres remanescentes    */
      /* que precisam ser removidos                 */
   if (teste != EOF) {
      nResto = LimpaBuffer();
   }

      /* Enquanto o valor retornado por scanf()  */
      /* no indicar que um valor vlido foi     */
      /* lido continua tentando obter esse valor */
   while(teste != 1) {
      printf("\a\n\t>>> O valor digitado e' invalido. "
                      "Tente novamente\n\t> " );
      goto inicio; /* No causa dano algum */
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf("\t>>> %d caracteres foram descartados\n", nResto);
   }

      /* O valor retornado certamente  vlido */
   return num;
}

/****
 *
 * RepresentacaoBinaria(): Apresenta no meio de sada a
 *                         representao binria de um valor
 *                         do tipo int
 *
 * Parmetros:
 *      numero (entrada): valor inteiro cuja representao
 *                        binria ser apresentada
 *
 * Retorno: Nada.
 *
 ****/
void RepresentacaoBinaria(int numero)
{
   int           i, bit, numeroDeBits;
   unsigned int  mascara; /* A mscara deve ser unsigned aqui */

       /* Cada byte contm um nmero de bits igual a CHAR_BIT */
   numeroDeBits  = CHAR_BIT*sizeof(int);

       /* Coloca 1 na posio mais  esquerda da  */
       /* mscara. Todos os outros bits tero 0s. */
   mascara = 0x1 << (numeroDeBits - 1);

     /* Exibe cada bit a partir da esquerda do nmero (parmetro) */
   for (i = 1; i <= numeroDeBits; i++) {
      bit = (numero & mascara) ? 1 : 0;

         /* Apresenta um bit na tela */
      printf("%d", bit);

         /* Separa bits em sequncias de oito para */
         /* melhor visualizao da representao   */
      if (i % 8 == 0)
         printf("  ");

         /* Move bit com valor 1 da mscara */
         /*  uma posio para a direita     */
      mascara >>= 1;
   }
}

/****
 *
 * ObtemMSBs(): Obtm os bits mais significativos de uma chave
 *
 * Parmetros:
 *      chave (entrada) - a chave
 *      nBits (entrada) - o nmero de bits mais significativos
 *                       que sero extrados da chave
 *
 * Retorno: Um valor inteiro que representa esses bits
 *
 ****/
int ObtemMSBs(int chave, int nBits)
{
   int mascara = (1 << nBits) - 1;

   ASSEGURA(chave >= 0, "A chave nao pode ser negativa");

   printf("\nMascara: ");
   RepresentacaoBinaria(mascara);
   printf("(%d)\n", mascara);
   putchar('\n');

      /* Desloca a chave para a direita, um bit de */
      /* cada vez, at que seu valor seja menor do */
      /* que ou igual ao valor da mscara          */
   while (chave > mascara) {
     chave >>= 1;
     printf("\nChave: ");
     RepresentacaoBinaria(chave);
   }

   return chave;
}

/****
 *
 * main(): Testa a funo ObtemMSBs()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int chave, nBMS, bms;

   while (1) {
      printf("\n>>> Introduza um valor inteiro: ");
      chave = LeInteiro();

      if (chave <= 0) {
         break;
      }

      printf("\n>>> Representacao binaria de %d:\n\t", chave);
      RepresentacaoBinaria(chave);

      printf("\n\n>>> Bits mais significativos que deseja obter: ");
      nBMS = LeInteiro();

      if (nBMS <= 0 || nBMS > 8*sizeof(int))
         break;

      bms = ObtemMSBs(chave, nBMS);

      printf( "\n>>> Bits mais significativos de %d: %d\n\t",
              chave, bms );

      printf("\n>>> Representacao binaria de %d:\n\t", bms);
      RepresentacaoBinaria(bms);
   }

   return 0;
}

/******************* Exemplo de Execuo ******************

>>> Introduza um valor inteiro: 2568

>>> Representacao binaria de 2568:
        00000000  00000000  00001010  00001000

>>> Bits mais significativos que deseja obter: 4

>>> Bits mais significativos de 2568: 10

>>> Representacao binaria de 10:
        00000000  00000000  00000000  00001010
>>> Introduza um valor inteiro: 0

**********************************************************/
