/****
 *
 * Exemplo: Representao interna de valores inteiros
 *
 * Seo: B.9
 *
 * Descrio: Apresenta a representao interna de valores
 *            inteiros positivos introduzidos pelo usurio
 *
 ****/

#include <limits.h>
#include <stdio.h>
#include <stdlib.h>

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeInteiro(): L um nmero inteiro via teclado
 *              e deixa o buffer intacto
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero inteiro lido
 *
 ****/
int LeInteiro(void)
{
   int num, /* O nmero lido */
       teste, /* Valor retornado por scanf() */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Desvia para c se o valor for invlido */
inicio:
      /* Tenta ler um valor vlido */
   teste = scanf("%d", &num);

      /* Se no ocorreu erro de leitura ou de final */
      /* de arquivo, h caracteres remanescentes    */
      /* que precisam ser removidos                 */
   if (teste != EOF) {
      nResto = LimpaBuffer();
   }

      /* Enquanto o valor retornado por scanf()  */
      /* no indicar que um valor vlido foi     */
      /* lido continua tentando obter esse valor */
   while(teste != 1) {
      printf("\a\n\t>>> O valor digitado e' invalido. "
                      "Tente novamente\n\t> " );
      goto inicio; /* No causa dano algum */
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf("\t>>> %d caracteres foram descartados\n", nResto);
   }

      /* O valor retornado certamente  vlido */
   return num;
}

/****
 *
 * RepresentacaoBinaria(): Apresenta no meio de sada a
 *                         representao binria de um valor
 *                         do tipo int
 *
 * Parmetros:
 *      numero (entrada): valor inteiro cuja representao
 *                        binria ser apresentada
 *
 * Retorno: Nada.
 *
 ****/
void RepresentacaoBinaria(int numero)
{
   int           i, bit, numeroDeBits;
   unsigned int  mascara; /* A mscara deve ser unsigned */

       /* Cada byte contm um nmero de bits igual a CHAR_BIT */
   numeroDeBits  = CHAR_BIT*sizeof(int);

       /* Coloca 1 na posio mais  esquerda da  */
       /* mscara. Todos os outros bits tero 0s. */
   mascara = 0x1 << (numeroDeBits - 1);

     /* Exibe cada bit a partir da esquerda */
   for (i = 1; i <= numeroDeBits; i++) {
      bit = (numero & mascara) ? 1 : 0;

         /* Apresenta um bit na tela */
      printf("%d", bit);

         /* Separa sequncias de bytes com espaos em branco */
         /* para melhor visualizao da representao        */
      if (i % CHAR_BIT == 0)
         printf("  ");

         /* Move bit com valor 1 da mscara */
         /* uma posio para a direita     */
      mascara >>= 1;
   }
}

/****
 *
 * main(): Testa a funo RepresentacaoBinaria()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int chave;

      /* Apresenta o programa ao usurio */
      /* e explica seu funcionamento     */
   printf( "\n\t>>> Este programa le um numero inteiro"
           "\n\t>>> e apresenta-o na tela. Um valor"
           "\n\t>>> negativo ou zero encerra o programa.\n" );

   while (1) {
      printf("\n\n>>> Introduza um valor inteiro: ");
      chave = LeInteiro();

      if (chave <= 0)
         break;

      printf("\n>>> Representacao binaria de %d:\n\t", chave);
      RepresentacaoBinaria(chave);
   }

   printf("\n>>> Bye, bye\n");

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa le um numero inteiro
        >>> e apresenta-o na tela. Um valor
        >>> negativo ou zero encerra o programa.


>>> Introduza um valor inteiro: 345

>>> Representacao binaria de 345:
        00000000  00000000  00000001  01011001

>>> Introduza um valor inteiro: -12

>>> Bye, bye

**********************************************************/
