/****
 *
 * Exemplo: Funes de Disperso Prontas: Funo de Disperso
 *          JOAAT
 *
 * Seo: B.1
 *
 * Descrio: Implementao da funo de disperso JOAAT
 *
 ****/

#include <stdio.h>

/****
 *
 * DispersaoJOAAT(): Implementao da funo de disperso JOAAT
 *
 * Parmetros:
 *      chave (entrada) - a chave cujo valor de disperso
 *                        ser calculado
 *
 * Retorno: O valor de disperso calculado
 *
 * Observao: Esta funo no testa se o ponteiro que
 *             representa o string  NULL (como deveria)
 *
 ****/
unsigned int DispersaoJOAAT(const char *chave)
{
   unsigned int dispersao = 0;

   while (*chave) {
      dispersao += *chave++;
      dispersao += (dispersao << 10);
      dispersao ^= (dispersao >> 6);
   }

   dispersao += (dispersao << 3);
   dispersao ^= (dispersao >> 11);
   dispersao += (dispersao << 15);

   return dispersao;
}

/****
 *
 * main(): Testa a funo DispersaoJOAAT()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   printf( "\n   >>>> Usando Dispersao de Jenkins <<<<\n");
   printf( "\n>>> Dispersao de \"bola\" = %u",
           DispersaoJOAAT("bola") );
   printf( "\n>>> Dispersao de \"loba\" = %u\n",
           DispersaoJOAAT("loba") );

   return 0;
}

/****************** Resultado do Programa *****************

   >>>> Usando Dispersao de Jenkins <<<<

>>> Dispersao de "bola" = 1624074313
>>> Dispersao de "loba" = 1460191399

**********************************************************/
