/****
 *
 * Exemplo: Funes de Disperso Prontas: Funes de Disperso
 *          DJB e DJB2
 *
 * Seo: B.2
 *
 * Descrio: Implementao da funo de disperso JOAAT
 *
 ****/

#include <stdio.h>

/****
 *
 * DispersaoDJB(): Implementao da funo de disperso DJB
 *
 * Parmetros:
 *      chave (entrada) - a chave cujo valor de disperso
 *                        ser calculado
 *
 * Retorno: O valor de disperso calculado
 *
 * Observao: Esta funo no testa se o ponteiro que
 *             representa o string  NULL (como deveria)
 *
 ****/
unsigned int DispersaoDJB(const char *chave)
{
   unsigned int dispersao = 0;

   while (*chave)
      dispersao = 33*dispersao + *chave++;

   return dispersao;
}

/****
 *
 * DispersaoDJB2(): Implementao da funo de disperso DJB2
 *
 * Parmetros:
 *      chave (entrada) - a chave cujo valor de disperso
 *                        ser calculado
 *
 * Retorno: O valor de disperso calculado
 *
 * Observao: Esta funo no testa se o ponteiro que
 *             representa o string  NULL (como deveria)
 *
 ****/
unsigned int DispersaoDJB2(const char *chave)
{
   unsigned int dispersao = 0;

   while (*chave)
      dispersao = 33*dispersao ^ *chave++;

   return dispersao;
}

/****
 *
 * main(): Testa as funes DJB() e DJB2()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main()
{
   printf( "\n   >>>> Usando Dispersao DJB <<<<\n");
   printf( "\n>>> Dispersao de \"bola\" = %u",
           DispersaoDJB("bola") );
   printf( "\n>>> Dispersao de \"loba\" = %u\n",
           DispersaoDJB("loba") );

   printf( "\n   >>>> Usando Dispersao DJB2 <<<<\n");
   printf( "\n>>> Dispersao de \"bola\" = %u",
           DispersaoDJB2("bola") );
   printf( "\n>>> Dispersao de \"loba\" = %u\n",
           DispersaoDJB2("loba") );

   return 0;
}

/****************** Resultado do Programa *****************

   >>>> Usando Dispersao DJB <<<<

>>> Dispersao de "bola" = 3646366
>>> Dispersao de "loba" = 4005406

   >>>> Usando Dispersao DJB2 <<<<

>>> Dispersao de "bola" = 3565120
>>> Dispersao de "loba" = 3763648

**********************************************************/
