/****
 *
 * Ttulo: Checando Balanceamento AVL
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 21/03/2017
 * ltima modificao: 21/03/2017
 *
 * Descrio: Verifica se uma rvore binria apresenta
 *            balanceamento AVL
 *
 * Entrada: No h
 *
 * Sada: Uma mensagem indicando se a rvore binria
 *        construda pelo programa tem balanceamento AVL
 *
 * OBSERVAO: Note que este programa no testa se uma
 *             rvore binria  uma rvore binria DE BUSCA
 *
 ****/

/*********************** Includes *************************/

#include<stdio.h>
#include<stdlib.h>

   /* Macro utilizada em tratamento */
   /* de condies de exceo       */
#define ASSEGURA(condicao, msg) if (!(condicao)) {\
                                   fprintf(stderr, "\n%s\n", msg);\
                                   exit(1); \
                                }

/****************** Definies de Tipos *******************/

typedef int tConteudo; /* Tipo do contedo efetivo de cada n */

   /* Tipo de n e tipo de ponteiro para n de uma rvore binria  */
typedef struct rotNoABB {
           struct rotNoABB *esquerda;
           tConteudo        conteudo;
           struct rotNoABB *direita;
        } tNoArvoreBB, *tArvoreBB;

/****************** Definies de Macros ******************/

   /* Calcula o maior de dois valores  */
#define MAIOR_VALOR(a, b) (((a) > (b)) ? (a) : (b))

   /* Calcula o valor absoluto de um nmero */
#define ABS(a) (((a) >= 0) ? (a) : (-(a)))

/***************** Definies de Funes ******************/

/****
 *
 * AlturaArvoreBB2(): Calcula a altura de uma rvore binria
 *
 * Parmetros:
 *      raiz (entrada) - ponteiro para a raiz da rvore
 *
 * Retorno: A altura da rvore binria cuja raiz  recebida como parmetro
 *
 ****/
int AlturaArvoreBB2(tNoArvoreBB *raiz)
{
      /* Se a rvore estiver vazia, sua altura  zero */
   if (!raiz)
      return 0;

      /* Se a rvore no estiver vazia, calcula-se a */
      /* maior altura de suas subrvore e soma-se um */
   return 1 + MAIOR_VALOR( AlturaArvoreBB2(raiz->esquerda),
                           AlturaArvoreBB2(raiz->direita) );
}

/****
 *
 * EhArvoreAVL(): Verifica se uma rvore binria tem balanceamento AVL
 *
 * Parmetros:
 *      raiz (entrada) - ponteiro para a raiz da rvore
 *
 * Retorno: 1, se a rvore tiver balanceamento AVL; 0, em caso contrrio
 *
 * OBSERVAO: Esta funo no testa a rvore  uma rvore
 *             binria DE BUSCA
 *
 ****/
int EhArvoreAVL(tNoArvoreBB *raiz)
{
   int altEsquerda, /* Altura de uma subrvore esquerda */
       altDireita;  /* Altura de uma subrvore direita  */

      /* Se a rvore estiver vazia, ela  AVL */
   if (!raiz)
      return 1;

      /* Obtm as alturas das subrvores esquerda e direita */
   altEsquerda = AlturaArvoreBB2(raiz->esquerda);
   altDireita = AlturaArvoreBB2(raiz->direita);

      /* A rvore ser AVL se sua raiz tiver balanceamento  */
      /* AVL e suas subrvores esquerda e direita forem AVL */
   if ( ABS(altEsquerda - altDireita) <= 1 && EhArvoreAVL(raiz->esquerda) &&
        EhArvoreAVL(raiz->direita) )
    return 1;

      /* Se ainda no houve retorno, a rvore no  AVL */
   return 0;
}

/****
 *
 * NoNovo(): Cria um n de uma rvore binria com o contedo
 *           especificado e subrvores vazias
 *
 * Parmetros:
 *      conteudo (entrada) - o contedo efetivo do n
 *
 * Retorno: Endereo do n criado
 *
 ****/
tNoArvoreBB *NoNovo(tConteudo conteudo)
{
   tNoArvoreBB *pNo = malloc(sizeof(tNoArvoreBB));

      /* Verifica se o n foi alocado */
   ASSEGURA(pNo, "Erro: Um no' da arvore nao foi alocado")

      /* Recheia o n */
   pNo->conteudo = conteudo;
   pNo->esquerda = pNo->direita = NULL;

   return pNo;
}

/****
 *
 * main(): Cria um rvore binria e testa a funo EhArvoreAVL()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   tNoArvoreBB *raiz = NoNovo(1); /* Cria a raiz da rvore */

      /* Insere os demais ns na rvore */
   raiz->esquerda = NoNovo(2);
   raiz->direita = NoNovo(3);
   raiz->esquerda->esquerda = NoNovo(4);
   raiz->esquerda->direita = NoNovo(5);
   raiz->esquerda->esquerda->esquerda = NoNovo(6);

      /* Verifica se a rvore tem balanceametno AVL */
   if (EhArvoreAVL(raiz))
      printf("\n\t>>> A arvore e' AVL\n");
   else
      printf("\n\t>>> A arvore NAO e' AVL\n");

   return 0;
}

/****************** Resultado do Programa *****************

        >>> A arvore NAO e' AVL

**********************************************************/
