/****
 *
 * Descrio: Escreve os registros do arquivo binrio
 *            "MiniCensoMEC2012.bin" num arquivo de texto
 *
 * Observaes:
 *     1. Este programa  til em depurao de rvores
 *        multidirecionais implementadas em arquivo.
 *
 *     2. Este programa no contm tratamento de exceo ou
 *        comentrios adequados. Ou pior, muitos comentrios
 *        no correspondem aquilo que  comentado.
 *
 *     3. IMPORATANTE: No utilize este programa com o
 *                     arquivo "CensoMEC2012.bin" (completo)
 *                     ou voc ir precisar comprar um novo HD.
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 15/03/2017
 * ltima modificao: 15/03/2017
 *
 * Entrada: Arquivo cujo nome  especificado pela constante
 *          NOME_ARQ_REG
 *
 * Sada: Arquivo cujo nome  especificado pela constante
 *        NOME_ARQ_TXT
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>

/***************** Constantes Simblicas ******************/

#define TAM_CODIGO_ALUNO  13 /* Nmero de dgitos num   */
                             /* cdigo de aluno */
#define TAM_ALUNO_CURSO   8 /* Nmero de dgitos num         */
                            /* cdigo de aluno/curso (chave) */
#define MAX_NOME_IES     200 /* Tamanho mximo de */
                             /* um nome de IES    */
#define MAX_NOME_CURSO   200 /* Tamanho mximo de */
                             /* um nome de curso  */

#define MAX_UF_NASCIMENTO 30 /* Tamanho mximo   */
                             /* de um nome de UF */

   /* Tamanho do acrscimo do bloco usado para conter */
   /* uma linha a cada chamada de realloc()           */
#define TAMANHO_BLOCO 256

   /* Nome do arquivo de registros */
#define NOME_ARQ_REG     "MiniCensoMEC2012.bin"

   /* Nome do arquivo de texto contendo registros */
#define NOME_ARQ_TXT     "RegistrosTeste.txt"

/****************** Definies de Tipos *******************/

typedef struct {
           long codigoAluno;
           int  codAlunoCurso;
           char nomeIES[MAX_NOME_IES];
           char nomeCurso[MAX_NOME_CURSO];
           char sexoAluno;
           int  idadeAluno;
           char UFNascimento[MAX_UF_NASCIMENTO];
           int  anoIngresso;
         } tRegistroMEC;

typedef tRegistroMEC tRegistro;

/****
 *
 * ExibeArrayDeChars(): Exibe na tela um array de elementos
 *                      do tipo char que no  string
 *
 * Parmetros:
 *      *nome (entrada) - string que ser exibido antes do array
 *       ar (entrada) - endereo do array
 *       tam (entrada) - tamanho do array
 *
 * Retorno: Nada
 *
 ****/
static void ExibeArrayDeChars(FILE *stream, const char *nome, const char *ar,
                              int tam)
{
   fputc('\n', stream);

   if (*nome) {
      fprintf(stream, "%s: ", nome);
   }

   for (int i = 0; i < tam && *ar; ++i) {
      fputc(ar[i], stream);
   }
}

/****
 *
 * ExibeRegistro(): Exibe na tela um registro do tipo tRegistro
 *
 * Parmetros: *registro (entrada) - registro que ser exibido
 *
 * Retorno: Nada
 *
 ****/
void ExibeRegistro(const tRegistro *registro, FILE *stream)
{
   fprintf(stream, "Codigo do aluno: %lld", registro->codigoAluno);

   fprintf(stream, "\nCodigo do curso: %d", registro->codAlunoCurso);

   ExibeArrayDeChars(stream, "Nome da IES", registro->nomeIES,
                     MAX_NOME_IES);

   ExibeArrayDeChars(stream, "Nome do curso", registro->nomeCurso,
                     MAX_NOME_CURSO);

   fprintf(stream, "\nSexo do aluno: %s",
          registro->sexoAluno ? "Masculino" : "Feminino");

   fprintf(stream, "\nIdade do aluno: %d", registro->idadeAluno);

   ExibeArrayDeChars(stream, "UF de nascimento",
                     registro->UFNascimento, MAX_UF_NASCIMENTO);

   fprintf(stream, "\nAno de ingresso: %d", registro->anoIngresso);
}

/****
 *
 * main(): Escreve o contedo de um arquivo binrio
 *         num arquivo de texto
 *
 * Parmetros: Nenhum
 *
 * Retorno: 0
 *
 ****/
int main(void)
{
   int     i = 0;
   tRegistroMEC  reg;
   FILE   *streamReg = fopen(NOME_ARQ_REG, "rb"),
          *streamTxt = fopen(NOME_ARQ_TXT, "w");

   while (1) {
      fread(&reg, sizeof(reg), 1, streamReg);

      if (feof(streamReg) || ferror(streamReg)) {
         break;
      }

      fprintf(streamTxt, "%d.\n\n", i++);

      ExibeRegistro(&reg, streamTxt);

      fprintf(streamTxt, "\n\n----------\n\n");
   }

   return 0;
}
