/****
 *
 * Arquivo ArvoreB.h
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 25/04/2004
 * ltima modificao: 21/07/2008
 *
 * Descrio: Interface das funes que implementam operaes
 *            usando rvores de busca multidirecional em arquivo
 *
 ****/

#ifndef _ArvoreB_H_  /* Previne incluso mltipla */
#define _ArvoreB_H_  /* deste arquivo             */

/*********************** Includes *************************/

#include <stdio.h>     /* Tipo FILE */
#include "Registros.h" /* Interface do mdulo Registros */

/************ Definies de Tipos e Constantes ************/

   /* Posio nula (invlida) em arquivo */
#define POSICAO_NULA -1

   /* Tipo do par chave/ndice armazenado nos ns */
typedef struct  {
          tChave chave;  /* Chave do registro              */
          int    indice; /* Posio do registro no arquivo */
} tChaveIndice;

      /***                               ***/
      /* Dimensionamento do grau da rvore */
      /***                               ***/

   /* Tamanho do bloco lido/escrito em disco */
#define TB  4096

   /* Tamanho de um par chave/ndice */
#define TCI  sizeof(tChaveIndice)

   /* Tamanho de um filho e do inteiro */
   /* que representa o grau do n      */
#define TI   sizeof(int)

   /* Clculo do grau da rvore */
//#define G  ((TB + TCI - TI)/(TCI + TI) - 1)
#define G 6

      /***                                      ***/
      /* Fim do dimensionamento do grau da rvore */
      /***                                      ***/

   /* Tipo de um n de uma rvore B */
typedef struct {
           int          nFilhos; /* O nmero de filhos do n */
           tChaveIndice chaves[G - 1];/* array contendo chaves  */
                                      /* e ndices de registros */
           int          filhos[G];/* array contendo as posies */
                                  /*  dos filhos no arquivo */
                                  /* que contm a rvore    */
        } tNoMultiMS, *tArvoreMultiMS;

/************************* Aluses ************************/

extern int LeRaizB(const char *nomeArqRaiz);
extern void EscreveRaizB(const char *nomeArqRaiz, int raiz);
extern int BuscaB(FILE *stream, int raiz, tChave chave);
extern int InsereB(int *posicaoRaiz, tChaveIndice *chaveEIndice, FILE *streamArvore);
extern int RemoveChaveB( FILE *streamArvore, int *raiz, tChave chave );
extern void CaminhamentoInfixoB(FILE *streamArvore, int posNo, FILE *streamChaves);
extern tChaveIndice MenorChaveB(FILE *streamArvore, int raiz);
extern tChaveIndice MaiorChaveB(FILE *streamArvore, int raiz);
extern int NumeroDeNosB(FILE *stream);
extern int NumeroDeChavesB(FILE *stream);
extern int AlturaB(FILE *stream, int posNo);

#endif

