/****
 *
 * Arquivo Dispersao.c
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 07/08/2017
 * ltima modificao: 25/05/2018
 *
 * Descrio: Implementao de funes de disperso
 *
 ****/

#include <stdio.h>     /* printf()         */
#include <stdlib.h>    /* srand() e rand() */
#include <time.h>      /* time()           */
#include <math.h>      /* sqrt(), modf() e floor() */
#include "Dispersao.h" /* Interface deste mdulo   */

/***************** Funes de Disperso *******************/

unsigned int DispersaoModular(const char *chave)
{
   unsigned int dispersao;

   dispersao = atoi(chave);

   return dispersao;
}

unsigned int DispersaoAditiva(const char *chave)
{
   unsigned int dispersao = 0;

   while (*chave) {
       dispersao += *chave++;
   }

   return dispersao;
}

unsigned int DispersaoPolinomial(const char *chave)
{
   unsigned int dispersao = 0,
                base = 2,
                primo = 31;

   while (*chave) {
      dispersao = (dispersao*base + *chave++)%primo;
   }

   return dispersao;
}

unsigned int DispersaoMultiplicativa(const char *str)
{
   unsigned int dispersao = 0;
   double       a = (sqrt(5) - 1)/2,
                pInteira,
                pFracionaria;
   unsigned int chave,
                tamTabela = 13;

   chave = atoi(str);

   pFracionaria = modf(chave*a, &pInteira);

   dispersao = (unsigned int) floor(tamTabela*pFracionaria);

   return dispersao;
}

unsigned int DispersaoRotativa(const char *chave)
{
   unsigned int dispersao = 0;

   while (*chave) {
       dispersao = (dispersao << 4) ^
                   (dispersao >> 28) ^ (*chave++);
   }

   return dispersao;
}

unsigned int DispersaoMistura(const char *chave)
{
   unsigned mascara = 0xFF000000;
   char     octetos[4];
   int      i, xor1, xor2;

   for (i = 0; i < 4; ++i) {
      octetos[i] = (*chave & mascara) >> ((3 - i)*8);
      mascara = mascara >> 8;
   }

   xor1 = octetos[0] ^ octetos[3];
   xor2 = octetos[1] ^ octetos[2];

   return xor1 ^ xor2;
}

