/****
 *
 * Arquivo DispersaoExt.h
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 04/08/2017
 * ltima modificao: 26/08/2017
 *
 * Descrio: Iterface das funes que implementam operaes
 *            de busca, insero e remoo usando disperso
 *            extensvel em memria secundria
 *
 ****/

#ifndef _DispersaoExt_H_ /* Previne incluso mltipla */
#define _DispersaoExt_H_

#include "Registros.h" /* Tipos tRegistroMEC e tChave */

/************ Definies de Tipos e Constantes ************/

   /* Posio nula (invlida) em arquivo */
#define POSICAO_NULA -1

#define TB 4096 /* Tamanho do bloco lido ou escrito  */
#define TR (int) sizeof(tChaveIndice) /* Tamanho de um registro   */
                                      /* armazenado nos coletores */
#define TI (int) sizeof(int) /* Tamanho de um campo do tipo int */

   /* Clculo do nmero mximo de elementos num coletor */
#define M ((TB - (2*TI))/TR - 1)
//#define M 100

   /* Tipo dos elementos que sero armazenados na tabela */
typedef struct {
           tChave chave;
           long   indice;
         } tChaveIndice;

   /* Tipo de um coletor */
typedef struct {
           tChaveIndice elementos[M]; /* Array de elementos  */
           int          nElementos;   /* Nmero de elementos */
                                      /* num coletor         */
           int          profLocal;    /* Profundidade local  */
        } tColetor;

   /* Tipo de uma tabela de busca com disperso extensvel */
typedef struct {
           int *dir;          /* Diretrio            */
           int  profGlobal;   /* Profundidade global  */
           int  tamDir;       /* Tamanho do diretrio */
           int  nEl; /* Nmero de elementos na tabela */
        } tTabelaDExt;

/************************* Aluses ************************/

extern void LeColetor(FILE *stream, int pos, tColetor *coletor);
extern void EscreveColetor( FILE *stream, int pos,
                            const tColetor *coletor );
extern void IniciaTabelaExt(tTabelaDExt *tabela, FILE *stream);
extern int BuscaExt( FILE *stream, const tTabelaDExt *tab,
                     tChave chave );
extern void InsereExt( tTabelaDExt *tab, const tChaveIndice *item,
                       FILE *stream );
extern int RemoveExt(tTabelaDExt *tab, tChave chave, FILE *stream);
extern int NRegistrosTabExt( const tTabelaDExt *tabela,
                             FILE *stream );
extern int NColetores(FILE *stream);
extern int NColetoresVazios(FILE *stream);
extern int NColetoresRepletos(FILE *stream);
extern int NColetoresOcupados(FILE *stream);
extern int ProfLocalMin(FILE *stream);
extern int ProfLocalMax(FILE *stream);

#endif
