/****
 *
 * Exemplo: Separando um String em Partes (Tokens): strtok()
 *
 * Seo: 9.10.1
 *
 ****/

#include <stdio.h>
#include <string.h>

/****
 *
 * ObtemTokens(): Separa um string em partes (tokens)
 *
 * Parmetros:
 *     str (entrada/sada) - o string que ser separado
 *                           ou NULL para continuar uma
 *                           separao anterior
 *     sep (entrada) - string contendo os separadores de tokens
 *
 * Retorno: Endereo do prximo token do string, se este
 *          existir, ou NULL, em caso contrrio
 *
 * Observao: Esta funo NO  funcionalmente equivalente 
 *             funo strtok() da biblioteca padro de C
 *
****/
char *ObtemTokens(char *str, char const *sep)
{
   static char *proximoToken; /* Aponta para o prximo */
                              /* token, se ele existir */
   char        *s, /* Apontar para o string no qual a */
                   /* busca pelo token ser efetuada   */
               *inicio = NULL; /* Guardar o incio */
                               /* do token corrente */

      /* Se 'str' no for um ponteiro nulo, o prximo  */
      /* token ser obtido a partir do incio de 'str' */
   if (str)
      proximoToken = str;

      /* Se 'proximoToken' for um ponteiro nulo, o */
      /* string ora explorado no tem mais tokens  */
   if (!proximoToken)
      return NULL; /* No h mais token nesse string */

      /* Obtm o endereo do primeiro separador */
      /* encontrado em 'proximoToken'           */
   s = strpbrk(proximoToken, sep);

      /* Verifica se foi encontrado algum separador */
      /* no string ora sendo explorado              */
   if (s) {
         /* Termina o token corrente na posio em */
         /* que se encontra o separador encontrado */
      *s = '\0';

         /* Guarda o incio do token corrente */
      inicio = proximoToken;

         /* O prximo token comear no primeiro */
         /* caractere aps o separador           */
      proximoToken = ++s;
   } else
      if (*proximoToken) {
            /* No foi encontrado nenhum separador, mas o string  */
            /* corrente no  vazio e seu endereo ser retornado */
         inicio = proximoToken;

         /* No foi encontrado nenhum separador, de modo que */
         /* no haver mais nenhum token na prxima chamada  */
      proximoToken = NULL;
   }

   return inicio;
}

int main(void)
{
   char string[] = "um,dois,,quatro,cinco";
   char separadores[] = " ,"; /* Note o espao em branco */
   char *token;
   int  i = 0;

   printf("\nString a ser separado em tokens: "
          "\"%s\"\n", string);
   printf("\nOs tokens sao:\n");

   token = ObtemTokens(string, separadores);

   while (token) {
      printf("\tToken %d: %s\n", ++i,
             *token ? token : "<vazio>");
      token = ObtemTokens(NULL, separadores);
   }

   return 0;
}

/****************** Resultado do Programa *****************

String a ser separado em tokens: "um,dois,,quatro,cinco"

Os tokens sao:
        Token 1: um
        Token 2: dois
        Token 3: <vazio>
        Token 4: quatro
        Token 5: cinco

**********************************************************/
