/****
 *
 * Exemplo: Heap Binrio
 *
 * Seo: 10.2.6
 *
 * Descrio: Este programa demonstra uma implementao de heap
 *            binrio ascendente
 *
 * Entrada: Opes escolhidas por meio de menu
 *
 * Sada: Resultados das operaes escolhidas pelo usurio
 *
 ****/

/*********************** Includes *************************/

#include <time.h>    /* Funes de cronometragem */
#include <string.h>  /* Funes de processamento de strings */
#include <stdlib.h>  /* Funo exit() */
#include <stdio.h>   /* Funes de Entrada/Sada */
#include "Heap.h"    /* Operaes sobre heap binrio */
#include "Leitura.h" /* Funes de leitura */
#include "ASSEGURA.h" /* Macro ASSEGURA()       */

/***************** Constantes Simblicas ******************/

   /* Tamanho mnimo de um heap criado aleatoriamente */
#define TAMANHO_MIN_HEAP 10
   /* Tamanho mximo de um heap criado aleatoriamente */
#define TAMANHO_MAX_HEAP 20
   /* Maior valor armazenado num heap criado aleatoriamente */
#define MAIOR_VALOR_HEAP 50

/***************** Definies de Funes ******************/

/****
 *
 * ApresentaMenu(): Apresenta um menu de opes para o usurio
 *
 * Parmetros:
 *      opcoes[] (entrada) - array de strings contendo as
 *                           opes do usurio
 *      nOpcoes (entrada) - o nmero de opes
 *
 * Retorno: Nada
 *
 ****/
static void ApresentaMenu(const char *opcoes[], int nOpcoes)
{
   int i;

   printf("\nEscolha uma das opcoes a seguir:\n");

   for (i = 0; i < nOpcoes; ++i)
      printf("\n\t[%d] %s", i + 1, opcoes[i]);

   printf("\n\n\t>>> ");
}

/****
 *
 * NAleatorio(): Retorna um nmero aleatrio entre os valores
 *               recebidos como parmetros
 *
 * Parmetros:
 *      m, n (entrada) - valores que definem o intervalo de
 *                       nmeros sorteados
 *
 * Retorno: O nmero aleatrio gerado
 *
 * Observaes:
 *      1.  indiferente se m <= n ou n <= m
 *      2. Quando chamada pela primeira vez, esta funo
 *         alimenta o gerador de nmeros aleatrios
 *
 ****/
int NAleatorio(int m, int n)
{
   static int primeiraChamada = 1; /* Esta varivel checa se a  */
                                   /* funo est sendo chamada */
                                   /* pela primeira vez e deve  */
                                   /* ter durao fixa          */

      /* Se esta for a primeira chamada da funo, */
      /* alimenta o gerador de nmeros aleatrios  */
   if (primeiraChamada) {
         /* Alimenta o gerador de nmeros aleatrios */
      srand(time(NULL));

         /* A prxima chamada no ser mais a primeira */
      primeiraChamada = 0;
   }

      /* Leva em considerao o fato de */
      /* n poder ser menor do que m     */
   if (n < m) {
      return rand()%(m - n + 1) + n;
   }

      /* m  menor do que ou igual a n */
   return rand()%(n - m + 1) + m;
}

/****
 *
 * ComparaInts(): Compara dois elementos do tipo int
 *
 * Parmetros:
 *      *e1, *e2 (entrada) - os elementos
 *
 * Retorno:
 *     = 0, se *e1 == *e2
 *     < 0, se *e1 < *e2
 *     > 0, se *e1 > *e2
 *
 ****/
int ComparaInts(const void *e1, const void *e2)
{
   ASSEGURA(e1 && e2, "Elemento nulo recebido por ComparaInts()");

   return *(int *)e1 - *(int *)e2;
}

/****
 *
 * CriaTabela(): Cria um array de inteiros aleatoriamente
 *
 * Parmetros:
 *     nElementos (sada) - nmero de elementos na tabela criada
 *     min (entrada) - nmero mnimo de elementos na tabela criada
 *     max (entrada) - nmero mximo de elementos na tabela criada
 *     maiorElemento (entrada) - valor do maior elemento da tabela
 *
 * Retorno: ponteiro para a tabela criada
 *
 ****/
int* CriaTabela( int *nElementos, int min, int max, int maiorElemento )
{
   int *tabela, i;

      /* Gera nmero de ns da tabela */
   *nElementos = NAleatorio(min, max);

   ASSEGURA( *nElementos >= min && *nElementos <= max,
             "Numero de elementos gerado fora do intervalo.");

   tabela = calloc(*nElementos, sizeof(int));
   ASSEGURA(tabela, "Impossivel alocar a tabela.");

      /* Gera valores dos elementos entre 0 e maiorElemento */
   for (i = 0; i < *nElementos; ++i)
      tabela[i] = rand()%maiorElemento;

   return tabela;
}

/********************* Funo main() **********************/

/****
 *
 * main(): Testa operaes de heap binrio ascendente
 *
 * Parmetros: nenhum
 *
 * Retorno: 0
 *
 ****/
int main(void)
{
   tHeap       heap, *pHeap;
   int         nElementos,
               item,
               op,
              *array;
   const char* opcoes[] = { "Insere um elemento",
                            "Apresenta o menor elemento",
                            "Remove o menor elemento",
                            "Cria heap com N elementos aleatorios",
                            "Remove todos os elementos",
                            "Exibe o conteudo do heap",
                            "Encerra o programa"
                          };

   IniciaHeap(&heap);

   pHeap = &heap;

   while (1) {
      ApresentaMenu(opcoes, sizeof(opcoes)/sizeof(opcoes[0]));
      op = LeOpcao("1234567");

         /* Verifica se o usurio quer encerrar o programa */
      if (op == '7')
         break;

      switch (op) {
         case '1':
            printf("\n>>> Digite o valor a ser inserido: ");
            item = LeInteiro();
            InsereEmHeap(&heap, item, ComparaInts);

            break;

         case '2':
            if (HeapVazio(pHeap))
               printf("\n>>> O heap esta' vazio\n");
            else {
               item = ObtemMinimoHeap(&heap);
               printf("\n>>> Menor elemento do heap: %d\n", item);
            }
            break;

         case '3':
            if (HeapVazio(pHeap))
               printf("\n>>> O heap esta' vazio\n");
            else {
               item = RemoveMinHeap(pHeap, ComparaInts);
               printf( "\n>>> Elemento removido do heap: %d\n",
                       item );
            }
            break;

         case '4':
            DestroiHeap(pHeap);
            array = CriaTabela( &nElementos, TAMANHO_MIN_HEAP,
                                TAMANHO_MAX_HEAP, MAIOR_VALOR_HEAP );
            pHeap = ConstroiHeap( array, nElementos,
                                  ComparaInts );
            printf( "\n>>> Criado um heap com %d elementos\n",
                    nElementos );
            break;

         case '5':
            while (!HeapVazio(pHeap)) {
               item = RemoveMinHeap(pHeap, ComparaInts);
               printf("\n>>> Elemento removido do heap: %d", item);
            }
            printf("\n>>> O heap foi esvaziado.\n");
            break;

         case '6':
            ExibeHeap(pHeap);
            break;

         default:
            printf("\nErro nao identificado");
            return 1;
            break;
      } /* switch */
   } /* while */

      /* Libera o espao ocupado pelo heap */
   DestroiHeap(pHeap);

      /* Despede-se do usurio */
   printf( "\n\t>>> Obrigado por usar este programa. Bye.\n\n");

   return 0;
}

/******************* Exemplo de Execuo ******************

Escolha uma das opcoes a seguir:

        [1] Insere um elemento
        [2] Apresenta o menor elemento
        [3] Remove o menor elemento
        [4] Cria heap com N elementos aleatorios
        [5] Remove todos os elementos
        [6] Encerra o programa

        >>> 2

>>> O heap esta' vazio

Escolha uma das opcoes a seguir:

        [1] Insere um elemento
        [2] Apresenta o menor elemento
        [3] Remove o menor elemento
        [4] Cria heap com N elementos aleatorios
        [5] Remove todos os elementos
        [6] Encerra o programa

        >>> 4

>>> Criado um heap com 16 elementos

Escolha uma das opcoes a seguir:

        [1] Insere um elemento
        [2] Apresenta o menor elemento
        [3] Remove o menor elemento
        [4] Cria heap com N elementos aleatorios
        [5] Remove todos os elementos
        [6] Encerra o programa

        >>> 3

>>> Elemento removido do heap: 0

Escolha uma das opcoes a seguir:

        [1] Insere um elemento
        [2] Apresenta o menor elemento
        [3] Remove o menor elemento
        [4] Cria heap com N elementos aleatorios
        [5] Remove todos os elementos
        [6] Encerra o programa

        >>> 5

>>> Elemento removido do heap: 3
>>> Elemento removido do heap: 11
>>> Elemento removido do heap: 12
>>> Elemento removido do heap: 12
>>> Elemento removido do heap: 13
>>> Elemento removido do heap: 16
>>> Elemento removido do heap: 18
>>> Elemento removido do heap: 21
>>> Elemento removido do heap: 23
>>> Elemento removido do heap: 24
>>> Elemento removido do heap: 30
>>> Elemento removido do heap: 34
>>> Elemento removido do heap: 42
>>> Elemento removido do heap: 44
>>> Elemento removido do heap: 49
>>> O heap foi esvaziado.

Escolha uma das opcoes a seguir:

        [1] Insere um elemento
        [2] Apresenta o menor elemento
        [3] Remove o menor elemento
        [4] Cria heap com N elementos aleatorios
        [5] Remove todos os elementos
        [6] Encerra o programa

        >>> 6

        >>> Obrigado por usar este programa. Bye.

**********************************************************/
