/****
 *
 * Ttulo: A Lanchonete HeapBurger
 *
 * Seo 10.5.2
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 06/10/2017
 * ltima modificao: 06/10/2017
 *
 * Descrio: Simulao de eventos discretos de funcionamento de
 *            uma lanchonete
 *
 * Entrada: v. exemplo de execuo
 *
 * Sada: v. exemplo de execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>     /* Funes de Entrada/Sada */
#include <string.h>    /* Processamento de strings */
#include <stdlib.h>    /* Funes de alocao dinmica */
#include <time.h>      /* Funo time() para medida de tempo */
#include "Eventos.h"   /* Processamento de eventos    */
#include "Heap.h"      /* Operaes sobre heaps       */
#include "Macros.h"    /* Macros usadas pelo programa */

/***************** Definies de Funes ******************/

/****
 *
 * main(): Executa uma simulao da Lanchonete HeapBurger
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int         t = 0, /* Instante de chegada de */
                      /* um grupo de clientes   */
               tMax = 0, /* Instante limite para chegada */
               nEventos = 0, /* Nmero de grupos de clientes */
                             /* que chegam  lanchonete      */
               nPessoasGrupo, /* Nmero de pessoas num */
                              /* grupo de clientes     */
               nPessoasTotal = 0; /* Nmero total de pessoas */
                                  /* que chegam  lanchonete */
   tHeap       fila; /* Fila de eventos */
   tEvento    *evento; /* Ponteiro para um evento */
   tLanchonete lanchonete = { N_CADEIRAS, /* Nmero de cadeiras */
                              0.0         /* Faturamento        */
                            };

         /* Apresenta o programa */
   printf( "\n\t>>> Este programa simula o funcionamento da "
           "\n\t>>> lanchonete virtual HeapBurger\n\n" );

      /* Inicia a fila de eventos */
   IniciaHeap(&fila);

      /* Exibe informaes bsicas sobre a lanchonete */
   ApresentaHora(H_ABERTURA, 0);
   printf( "\tAbertura da lanchonete");
   printf( "\n\t\t\tNumero de cadeiras disponiveis: %d",
           N_CADEIRAS );
   printf( "\n\t\t\tPreco de um sanduiche BigHeap: R$%.2f\n",
           PRECO );
   ApresentaHora(H_ABERTURA, PERIODO);
   printf( "\tFechamento da lanchonete\n");

      /* Cria alguns eventos de chegada e armazena-os na fila */
   while (1) {
      t += IntervaloExponencial(INTERVALO_CHEGADA);

         /* Atualiza o tempo mximo de funcionamento */
      if (t > tMax)
         tMax = t;

         /* Verifica se o instante de ocorrncia do prximo */
         /* evento ser aps o limite de funcionamento      */
      if (tMax > PERIODO)
         break;

         /* Obtm o nmero de pessoas no */
         /* grupo do presente evento     */
      nPessoasGrupo = NAleatorio(1, MAX_PESSOAS_GRUPO);

         /* Atualiza o nmero total de pessoas */
      nPessoasTotal += nPessoasGrupo;

         /* Cria o evento de chegada */
      evento = CriaEvento(CHEGADA, t, nPessoasGrupo);

         /* Insere o evento recm-criado */
         /* na fila de eventos pendentes */
      InsereEmHeap(&fila, evento, ComparaEventos);

         /* Atualiza o nmero total de eventos */
      ++nEventos;
   }

   printf( "\n\t>>> Numero de grupos que chegaram na "
           "lanchonete: %d", nEventos);
   printf( "\n\t>>> Numero de clientes que chegaram na "
           "lanchonete: %d\n\n", nPessoasTotal);

      /* Executa a simulao */
   while (!HeapVazio(&fila)) {
      evento = RemoveMin(&fila, ComparaEventos);
      DespachaEvento(evento, &lanchonete, &fila);
   }

   printf( "\n\t>>> Faturamento da lanchonete: R$%.2f",
           lanchonete.faturamento );

   printf( "\n\t>>> Lucro da lanchonete: R$%.2f\n",
           MARGEM*lanchonete.faturamento );

   printf("\n\t>>> Fim da simulacao\n\n");

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa simula o funcionamento da
        >>> lanchonete virtual HeapBurger

        >>> 10:00       Abertura da lanchonete
                        Numero de cadeiras disponiveis: 10
                        Preco de um sanduiche BigHeap: R$8.00
        >>> 12:00       Fechamento da lanchonete

        >>> Numero de grupos que chegaram na lanchonete: 5
        >>> Numero de clientes que chegaram na lanchonete: 14

        >>> 10:08       4 fregueses chegam e sentam
        >>> 10:08       Atendido pedido de 1 sanduiche
        >>> 10:08       Atendido pedido de 3 sanduiches
        >>> 10:08       Atendido pedido de 2 sanduiches
        >>> 10:08       Atendido pedido de 3 sanduiches
        >>> 10:09       Grupo de 4 pessoas sai
        >>> 10:29       1 fregues chega e senta
        >>> 10:30       Atendido pedido de 3 sanduiches
        >>> 10:46       Grupo de 1 pessoa sai
        >>> 11:46       3 fregueses chegam e sentam
        >>> 11:51       Atendido pedido de 1 sanduiche
        >>> 11:51       Atendido pedido de 1 sanduiche
        >>> 11:51       Atendido pedido de 2 sanduiches
        >>> 11:47       2 fregueses chegam e sentam
        >>> 11:52       Atendido pedido de 3 sanduiches
        >>> 11:52       Atendido pedido de 2 sanduiches
        >>> 11:50       4 fregueses chegam e sentam
        >>> 11:50       Atendido pedido de 1 sanduiche
        >>> 11:50       Atendido pedido de 2 sanduiches
        >>> 11:50       Atendido pedido de 2 sanduiches
        >>> 11:50       Atendido pedido de 2 sanduiches
        >>> 11:51       Grupo de 3 pessoas sai
        >>> 11:55       Grupo de 4 pessoas sai
        >>> 11:58       Grupo de 2 pessoas sai

        >>> Faturamento da lanchonete: R$224.00
        >>> Lucro da lanchonete: R$67.20

        >>> Fim da simulacao

**********************************************************/
