/****
 *
 * Exemplo: SelectionSort
 *
 * Seo: 11.2.2
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>  /* Funes de entrada e sada     */
#include <stdlib.h> /* Funes de alocao de memria */
#include <time.h>   /* Funes de cronometragem       */
#include <string.h> /* memmove()                      */

/***************** Constantes Simblicas ******************/

   /* Nmero mnimo de elementos na tabela a ser ordenada */
#define MIN_EL   10
   /* Nmero mximo de elementos na tabela a ser ordenada */
#define MAX_EL   20

/****************** Definies de Macros ******************/

   /* Macro utilizada para verificar     */
   /* ocorrncia de condies de exceo */
#define ASSEGURA( x, msg ) if (!(x) ) {\
                              printf("\n%s", msg); \
                              exit(1); \
                           }

/************************* Aluses ************************/

extern void SelectionSort(int tabela[], int nElem);
extern int* CriaTabela(int *nElementos, int min, int max);
extern void ApresentaTabela(const int *tabela, int nElementos);
extern void LiberaTabela(int **tabela);
extern int EstaOrdenada(int tabela[], int nElementos);

/***************** Definies de Funes ******************/

/****
 *
 * TrocaGenerica(): Troca os valores de duas variveis do mesmo tamanho
 *
 * Parmetros:
 *      var1, var2 (entrada e sada) - endereos das variveis
 *                                     que sero trocadas
 *      tam (entrada) - tamanho das variveis que sero trocadas
 *
 * Retorno: 0, se no ocorrer erro; 1, em caso contrrio
 *
 ****/
static void TrocaGenerica(void *var1, void *var2, size_t tam)
{
   void *p;

   p = malloc(tam);
   ASSEGURA(p, "Impossivel trocar duas variaveis");

   memmove(p, var1, tam);
   memmove(var1, var2, tam);
   memmove(var2, p, tam);

   free(p);
}

/****
 *
 * SelectionSort(): Ordena uma tabela usando seleo direta
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser a ordenada
 *      nElem (entrada) - nmero de elementos na tabela
 *
 * Retorno: Nada
 *
 ****/
void SelectionSort(int tabela[], int nElem)
{
   int i, j, iMenor;

      /* A poro ordenada do array est entre os ndices 0 e i - 1, */
      /* enquanto que a poro desordenada est entre i e n - 1      */

      /* Obtm o menor elemento na parte desordenada e troca-o de posio */
      /* com o primeiro elemento da parte desordenada (que  i)           */
   for (i = 0; i < nElem - 1; ++i) {
         /* Supe que o primeiro elemento da parte desordenada  o menor */
      iMenor = i;

         /* Verifica se existe um elemento menor */
         /* do 'iMenor' na parte desordenada     */
      for (j = i + 1; j < nElem; ++j)
            /* Se o elemento corrente for menor do que */
            /* 'iMenor', ele passar a ser o menor     */
        if (tabela[j] < tabela[iMenor])
            iMenor = j; /* O novo mnimo passa a ser j */

         /* Se foi encontrado um elemento menor do que aquele que se encontra   */
         /* na posio i, trocam-se as posies desses elementos. Caso          */
         /* contrrio, o elemento na posio i j est em sua posio ordenada. */
      if (iMenor != i)
        TrocaGenerica(tabela + i, tabela + iMenor, sizeof(tabela[0]));
   }
}

/****
 *
 * CriaTabela(): Cria uma tabela (array de inteiros) aleatoriamente
 *
 * Parmetros:
 *      nElementos (sada) - nmero de elementos na
 *                           tabela criada
 *      min (entrada) - nmero mnimo de elementos
 *                      na tabela criada
 *      max (entrada) - nmero mximo de elementos na
 *                      tabela criada
 *
 * Retorno: Endereo da tabela criada
 *
 ****/
int* CriaTabela(int *nElementos, int min, int max)
{
   int *tabela, nElem, i, maiorElemento;

      /* Gera nmero de ns da tabela */
   nElem = (int) rand()%(max-min+1) + min;

   ASSEGURA( nElem >= min && nElem <= max,
             "Numero de elementos fora do intervalo" );

   tabela = calloc(nElem, sizeof(int));
   ASSEGURA( tabela, "Nao foi possivel alocar a tabela" );

      /* Isto no  necessrio e  feito apenas para */
      /* facilitar a visualizao dos resultados     */
   maiorElemento = 2*max;


      /* Gera valores dos elementos entre 0 e maiorElemento */
   for (i = 0; i < nElem; ++i)
      tabela[i] = (int) rand()%maiorElemento;

   *nElementos = nElem;

   return tabela;
}

/****
 *
 * ApresentaTabela(): Apresenta na tela a tabela recebida como
 *                    parmetro
 *
 * Parmetros:
 *      tabela (entrada) - a tabela cujos elementos sero exibidos
 *      nElementos (entrada) - nmero mnimo de elementos na tabela
 *
 * Retorno: Nada
 *
 ****/
void ApresentaTabela(const int *tabela, int nElementos)
{
   int i;

   printf("\nNumero de elementos: %d", nElementos);
   printf("\n{");

   for (i = 0; i < nElementos - 1; ++i)
      printf("%d, ", tabela[i]);

   printf("%d}", tabela[nElementos - 1]);
}

/****
 *
 * LiberaTabela(): Libera o espao ocupado por uma tabela
 *                 e torna nulo o ponteiro que a representa
 *
 * Parmetros:
 *      tabela (entrada/sada) - a tabela cujo espao ser
 *                               liberado
 *
 * Retorno: Nada
 *
 * NOTA: Se o ponteiro para o incio da tabela for nulo,
 *       nada ser feito; caso contrrio, assume-se que a
 *       tabela foi alocada dinamicamente
 *
 ****/
void LiberaTabela(int **tabela)
{
   if(*tabela) {
      free(*tabela);
      *tabela = NULL;
   }
}

/****
 *
 * EstaOrdenada(): Testa se uma tabela est ordenada em
 *                 ordem crescente
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser a testada
 *      nElem (entrada) - nmero de elementos na tabela
 *
 * Retorno: 1, se a tabela estiver ordenada;
 *          0, em caso contrrio
 *
 ****/
int EstaOrdenada(int tabela[], int nElementos)
{
   int i;

   for (i = 0; i < nElementos -1; ++i)
      if (tabela[i] > tabela[i+1])
         return 0;

   return 1;
}

/****
 *
 * main(): Testa ordenao usando SelectionSort
 *
 * Parmetros: Nenhum
 *
 * Retorno: 0, se no ocorrer erro; 1, em caso contrrio
 *
 ****/
int main(void)
{
   int *tabela; /* Ponteiro para as tabela utilizada */
   int  nElementos;

   srand((int) time(NULL));

   tabela = CriaTabela(&nElementos, MIN_EL, MAX_EL);

   printf("\n\n>>>>>>>>>>>>>>> SelectionSort <<<<<<<<<<<<<<<\n");

   printf("\n************ Antes da Ordenacao *************\n");
   ApresentaTabela(tabela, nElementos);

   SelectionSort(tabela, nElementos);

   if (!EstaOrdenada(tabela, nElementos)) {
      printf("\n\n\a>>> A tabela NAO foi ordenada <<<\n");
      return 1;
   }

   printf("\n\n************ Depois da Ordenacao ************\n");
   ApresentaTabela(tabela, nElementos);
   putchar('\n');

   LiberaTabela(&tabela);

   return 0;
}

/******************* Exemplo de Execuo ******************

>>>>>>>>>>>>>>> SelectionSort <<<<<<<<<<<<<<<

************ Antes da Ordenacao *************

Numero de elementos: 11
{18, 30, 22, 16, 33, 28, 22, 34, 15, 18, 29}

************ Depois da Ordenacao ************

Numero de elementos: 11
{15, 16, 18, 18, 22, 22, 28, 29, 30, 33, 34}

**********************************************************/
