/****
 *
 * Exemplo: MergeSort
 *
 * Seo: 11.3.2
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>  /* Funes de entrada e sada     */
#include <stdlib.h> /* Funes de alocao de memria */
#include <time.h>   /* Funes de cronometragem       */

/***************** Constantes Simblicas ******************/

   /* Nmero mnimo de elementos na tabela a ser ordenada */
#define MIN_EL   10
   /* Nmero mximo de elementos na tabela a ser ordenada */
#define MAX_EL   20

/****************** Definies de Macros ******************/

   /* Macro utilizada para verificar     */
   /* ocorrncia de condies de exceo */
#define ASSEGURA( x, msg ) if (!(x) ) {\
                              printf("\n%s", msg); \
                              exit(1); \
                           }

/************************* Aluses ************************/

extern void MergeSort(int tabela[], int nElem);
extern int* CriaTabela(int *nElementos, int min, int max);
extern void ApresentaTabela(const int *tabela, int nElementos);
extern void LiberaTabela(int **tabela);
extern int EstaOrdenada(int tabela[], int nElementos);

/***************** Definies de Funes ******************/

/****
 *
 * Intercala2Tabelas(): Faz a intercalao de duas subtabelas
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela contendo as duas subtabela que sero
 *                               intercaladas
 *      aux (entrada/sada) - tabela auxiliar que conter o resultado
 *      iniEsq (entrada) - incio da metade da esquerda
 *      iniDir (entrada) - incio da metade da direita
 *      finalDir (entrada) - final da metade da direita
 *
 * Retorno: Nada
 *
 ****/
static void Intercala2Tabelas( int tabela[], int aux[], int iniEsq, int iniDir,
                               int finalDir )
{
   int i, finalEsq, nElementos, posAux;

   finalEsq = iniDir - 1;
   posAux = iniEsq;
   nElementos = finalDir - iniEsq + 1;

      /* Intercala as duas metades e coloca o resultado na tabela auxiliar */
   while((iniEsq <= finalEsq) && (iniDir <= finalDir))
      if(tabela[iniEsq] <= tabela[iniDir])
         aux[posAux++] = tabela[iniEsq++];
      else
         aux[posAux++] = tabela[iniDir++];

      /* Copia o restante da primeira metade */
   while(iniEsq <= finalEsq)
      aux[posAux++] = tabela[iniEsq++];

      /* Copia o restante da segunda metade */
   while(iniDir <= finalDir)
      aux[posAux++] = tabela[iniDir++];

      /* Copia a tabela auxiliar de */
      /* volta na tabela original   */
   for(i = 0; i < nElementos; i++,finalDir--)
      tabela[finalDir] = aux[finalDir];
}

/****
 *
 * MergeSortAux(): Ordena uma tabela usando intercalao com uma
 *                 tabela auxiliar
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela a ser ordenada
 *      aux (entrada/sada) - tabela auxiliar
 *      esquerda (entrada) - incio da metade da esquerda da tabela
 *      direita (entrada) - incio da metade da direita da tabela
 *
 * Retorno: Nada
 *
 ****/
static void MergeSortAux(int *tabela, int *aux, int esquerda, int direita)
{
   int centro;

   if(esquerda < direita) {
      centro = esquerda + (direita - esquerda)/2;

         /* Ordena metade inferior da tabela */
      MergeSortAux(tabela, aux, esquerda, centro);
         /* Ordena metade superior da tabela */
      MergeSortAux(tabela, aux, centro + 1, direita);

         /* Intercala as duas metades da tabela na tabela auxiliar */
      Intercala2Tabelas(tabela, aux, esquerda, centro + 1, direita);
   }
}

/****
 *
 * MergeSort(): Ordena uma tabela usando ordenao por intercalao
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser a ordenada
 *      nElem (entrada) - nmero de elementos na tabela
 *
 * Retorno: Nada
 *
 ****/
void MergeSort(int tabela[], int nElem)
{
   int *aux;

      /* Aloca espao para a tabela auxiliar */
   aux = malloc(nElem*sizeof(int));
   ASSEGURA(aux, "Impossivel alocar tabela auxiliar");

      /* Chama MergeSortAux() para fazer o servio */
   MergeSortAux(tabela, aux, 0, nElem - 1);

      /* Libera o espao ocupado pela tabela auxiliar */
   free(aux);
}

/****
 *
 * CriaTabela(): Cria uma tabela (array de inteiros) aleatoriamente
 *
 * Parmetros:
 *      nElementos (sada) - nmero de elementos na
 *                           tabela criada
 *      min (entrada) - nmero mnimo de elementos
 *                      na tabela criada
 *      max (entrada) - nmero mximo de elementos na
 *                      tabela criada
 *
 * Retorno: Endereo da tabela criada
 *
 ****/
int* CriaTabela(int *nElementos, int min, int max)
{
   int *tabela, nElem, i, maiorElemento;

      /* Gera nmero de ns da tabela */
   nElem = (int) rand()%(max-min+1) + min;

   ASSEGURA( nElem >= min && nElem <= max,
             "Numero de elementos fora do intervalo" );

   tabela = calloc(nElem, sizeof(int));
   ASSEGURA( tabela, "Nao foi possivel alocar a tabela" );

      /* Isto no  necessrio e  feito apenas para */
      /* facilitar a visualizao dos resultados     */
   maiorElemento = 2*max;


      /* Gera valores dos elementos entre 0 e maiorElemento */
   for (i = 0; i < nElem; ++i)
      tabela[i] = (int) rand()%maiorElemento;

   *nElementos = nElem;

   return tabela;
}

/****
 *
 * ApresentaTabela(): Apresenta na tela a tabela recebida como
 *                    parmetro
 *
 * Parmetros:
 *      tabela (entrada) - a tabela cujos elementos sero
 *                         exibidos
 *      nElementos (entrada) - nmero mnimo de elementos na
 *                             tabela
 *
 * Retorno: Nada
 *
 ****/
void ApresentaTabela(const int *tabela, int nElementos)
{
   int i;

   printf("\nNumero de elementos: %d", nElementos);
   printf("\n{");

   for (i = 0; i < nElementos - 1; ++i)
      printf("%d, ", tabela[i]);

   printf("%d}", tabela[nElementos - 1]);
}

/****
 *
 * LiberaTabela(): Libera o espao ocupado por uma tabela
 *                 e torna nulo o ponteiro que a representa
 *
 * Parmetros:
 *      tabela (entrada/sada) - a tabela cujo espao ser
 *                               liberado
 *
 * Retorno: Nada
 *
 * NOTA: Se o ponteiro para o incio da tabela for nulo,
 *       nada ser feito; caso contrrio, assume-se que a
 *       tabela foi alocada dinamicamente
 *
 ****/
void LiberaTabela(int **tabela)
{
   if(*tabela) {
      free(*tabela);
      *tabela = NULL;
   }
}

/****
 *
 * EstaOrdenada(): Testa se uma tabela est ordenada em
 *                 ordem crescente
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser a testada
 *      nElem (entrada) - nmero de elementos na tabela
 *
 * Retorno: 1, se a tabela estiver ordenada;
 *          0, em caso contrrio
 *
 ****/
int EstaOrdenada(int tabela[], int nElementos)
{
   int i;

   for (i = 0; i < nElementos -1; ++i)
      if (tabela[i] > tabela[i+1])
         return 0;

   return 1;
}

/****
 *
 * main(): Testa ordenao usando MergeSort
 *
 * Parmetros: Nenhum
 *
 * Retorno: 0, se no ocorrer erro; 1, em caso contrrio
 *
 ****/
int main(void)
{
   int *tabela; /* Ponteiro para as tabela utilizada */
   int  nElementos;

   srand((int) time(NULL));

   tabela = CriaTabela(&nElementos, MIN_EL, MAX_EL);

   printf("\n\n>>>>>>>>>>>>>>>> MergeSort <<<<<<<<<<<<<<<<\n");

   printf("\n*********** Antes da Ordenacao ************\n");
   ApresentaTabela(tabela, nElementos);

   MergeSort(tabela, nElementos);

   if (!EstaOrdenada(tabela, nElementos)) {
      printf("\n\n\a>>> Erro: A tabela NAO foi ordenada <<<\n");
      return 1;
   }

   printf("\n\n*********** Depois da Ordenacao ***********\n");
   ApresentaTabela(tabela, nElementos);
   putchar('\n');

   LiberaTabela(&tabela);

   return 0;
}

/******************* Exemplo de Execuo ******************

>>>>>>>>>>>>>>>> MergeSort <<<<<<<<<<<<<<<<

*********** Antes da Ordenacao ************

Numero de elementos: 13
{35, 11, 24, 32, 28, 11, 22, 34, 6, 26, 15, 0, 38}

*********** Depois da Ordenacao ***********

Numero de elementos: 13
{0, 0, 6, 11, 11, 15, 22, 24, 26, 28, 32, 34, 35}

**********************************************************/
