/****
 *
 * Arquivo: Metodos.c
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/09/2017
 * ltima modificao: 16/09/2017
 *
 * Descrio: Implementao de mtodos de ordenao
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Funes de entrada e sada     */
#include <stdlib.h>   /* Funes de alocao de memria */
#include <time.h>     /* Funes de cronometragem       */
#include <string.h>   /* Funo memmove()               */
#include <limits.h>   /* Limites de inteiros            */
#include "Metodos.h"  /* Interface deste mdulo         */
#include "ASSEGURA.h" /* Macro ASSEGURA                 */

/******************** Funes de Locais *******************/

/****
 *
 * TrocaGenerica(): Troca os valores de duas variveis do mesmo tamanho
 *
 * Parmetros:
 *      var1, var2 (entrada e sada) - endereos das variveis
 *                                     que sero trocadas
 *      tam (entrada) - tamanho das variveis que sero trocadas
 *
 * Retorno: 0, se no ocorrer erro; 1, em caso contrrio
 *
 ****/
static void TrocaGenerica(void *var1, void *var2, size_t tam)
{
   void *p;

   p = malloc(tam);
   ASSEGURA(p, "Impossivel trocar duas variaveis");

   memmove(p, var1, tam);
   memmove(var1, var2, tam);
   memmove(var2, p, tam);

   free(p);
}

/*********************** BubbleSort ***********************/

/****
 *
 * BubbleSort(): Ordena uma tabela usando BubbleSort
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser a ordenada
 *      nElem (entrada) - nmero de elementos na tabela
 *
 * Retorno: Nada
 *
 ****/
void BubbleSort(int tabela[], int nElem)
{
   int i, ordenado = 0;

   while (!ordenado){
         /* Supe que a tabela est ordenada */
      ordenado = 1;

      for (i = 0; i < nElem - 1; i++)
            /* Compara elementos adjacentes */
         if (tabela[i] > tabela[i + 1]){
               /* Pelo menos um par de elementos */
               /* est fora de ordem             */
            ordenado = 0;

               /* Troca elementos que esto fora de ordem */
            TrocaGenerica(tabela + i, tabela + i + 1, sizeof(tabela[0]));
         }
   }
}

/********************** SelectionSort *********************/

/****
 *
 * SelectionSort(): Ordena uma tabela usando seleo direta
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser a ordenada
 *      nElem (entrada) - nmero de elementos na tabela
 *
 * Retorno: Nada
 *
 ****/
void SelectionSort(int tabela[], int nElem)
{
   int i, j, iMenor;

      /* A poro ordenada do array est entre os */
      /* ndices 0 e i - 1, enquanto que a poro */
      /* desordenada est entre i e n - 1         */

      /* Obtm o menor elemento na parte desordenada  */
      /* e troca-o de posio com o primeiro elemento */
      /* da parte desordenada (que  i)               */
   for (i = 0; i < nElem - 1; ++i) {
         /* Supe que o primeiro elemento  */
         /* da parte desordenada  o menor */
      iMenor = i;

         /* Verifica se existe um elemento menor */
         /* do 'iMenor' na parte desordenada     */
      for (j = i + 1; j < nElem; ++j)
            /* Se o elemento corrente for menor do que */
            /* 'iMenor', ele passar a ser o menor     */
        if (tabela[j] < tabela[iMenor])
            iMenor = j; /* O novo mnimo passa a ser j */

         /* Se foi encontrado um elemento menor do que aquele   */
         /* que se encontra na posio i, trocam-se as posies */
         /* desses elementos. Caso contrrio, o elemento na     */
         /* posio i j est em sua posio ordenada.          */
      if (iMenor != i)
        TrocaGenerica(tabela + i, tabela + iMenor, sizeof(tabela[0]));
   }
}

/********************** InsertionSort *********************/

/****
 *
 * InsertionSort(): Ordena uma tabela usando ordenao por
 *                  insero simples
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser a ordenada
 *      nElem (entrada) - nmero de elementos na tabela
 *
 * Retorno: Nada
 *
 ****/
void InsertionSort(int tabela[], int  nElem)
{
   int i, j, aux;

      /* Inicialmente, o array tabela[]  considerado */
      /* uma tabela ordenada. Ento, a cada iterao, */
      /* os elementos entre 0 e j estaro ordenados.  */

      /* Insere tabela[j] na parte ordenada da tabela */
   for (j = 1; j < nElem; ++j) {
      aux = tabela[j];

         /* Move cada elemento maior do */
         /* que aux uma posio adiante */
      for (i = j - 1; i >= 0 && aux < tabela[i]; --i)
         tabela[i + 1] = tabela[i];

      tabela[i + 1] = aux; /* Insere aux na posio correta */
   }
}

/*********************** QuickSort 1 **********************/

/****
 *
 * Quick1(): Ordena uma tabela usando o algoritmo QuickSort
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser ordenada
 *      inf (entrada) - limite inferior da tabela
 *      sup (entrada) - limite superior da tabela
 *
 * Retorno: Nada
 *
 * Observaes:
 *   1. Por simplicidade, i.e., com o objetivo didtico
 *      de demonstrar o mtodo esta funo  implementada
 *      recursivamente, mas evidentemente uma implementao
 *      no recursiva seria mais eficiente
 *   2. O piv  o primeiro elemento da tabela
 *
 ****/
static void Quick1(int tabela[], int inf, int sup)
{
   int i, j, pivo;

   if (inf >= sup)
      return; /* A tabela j deve estar ordenada */

      /* Nesta implementao, o piv  sempre */
      /* o primeiro elemento da tabela        */
   pivo = inf;

      /* Cria as parties */
   for (i = inf, j = sup; i < j; ) {
         /* Enquanto os elementos da tabela forem menores */
         /* do que o piv, avana-se o ndice inferior    */
      while (tabela[i] <= tabela[pivo] && i < sup)
         ++i;

         /* Enquanto os elementos da tabela forem maiores */
         /* do que o piv, recua-se o ndice superior     */
      while (tabela[j] > tabela[pivo])
         --j;

         /* Os elementos nos ndices i e j esto em parties */
         /* erradas e  preciso troc-los de lugar            */
      if (i < j)
         TrocaGenerica(tabela + i, tabela + j, sizeof(tabela[0]));
   }

      /* Coloca o piv em sua correta posio */
   TrocaGenerica(tabela + pivo, tabela + j, sizeof(tabela[0]));

      /* Ordena recursivamente a primeira metade da tabela */
   Quick1(tabela, inf, j - 1);

      /* Ordena recursivamente a segunda metade da tabela */
   Quick1(tabela, j + 1, sup);
}

/****
 *
 * QuickSort1(): Ordena uma tabela usando quick sort. Esta funo
 *               simplesmente prov uma interface para a funo
 *               Quick1() que realmente faz o servio.
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser a ordenada
 *      nElem (entrada) - nmero de elementos na tabela
 *
 * Retorno: Nada
 *
 ****/
void QuickSort1(int tabela[], int nElem)
{
      /* Simplesmente chama Quick() para fazer o servio */
   Quick1(tabela, 0, nElem - 1);
}

/*********************** QuickSort 2 **********************/

   /* Ponto de corte */
#define PONTO_DE_CORTE 3

/****
 *
 * Quick2(): Ordena uma tabela usando QuickSort com mediana de
 *           trs e ponto de corte
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser ordenada
 *      inf (entrada) - limite inferior da tabela
 *      sup (entrada) - limite superior da tabela
 *
 * Retorno: Nada
 *
 * Observaes:
 *   1. Por simplicidade, i.e., com o objetivo didtico
 *      de demonstrar o mtodo esta funo  implementada
 *      recursivamente, mas evidentemente uma implementao
 *      no recursiva seria mais eficiente
 *   2. O piv  escolhido usando a tcnica mediana de trs
 *   3. Quando o ponto de corte  atingido a tabela  ordenada
 *      usando InsertionSort
 *
 ****/
static void Quick2(int tabela[], int inf, int sup)
{
   int meio, pivo, i, j;

      /* Se o tamanho da tabela ficar abaixo do ponto de */
      /* corte, chama InsertionSort() para orden-la     */
   if (sup - inf + 1 <= PONTO_DE_CORTE)
      InsertionSort(tabela + inf, sup - inf + 1);
   else {
      /* Escolhe o piv usando mediana de trs. Alm */
      /* disso, ao final, os elementos nas posies  */
      /* inf, sup e meio estaro ordenados.          */

         /* Obtm o ndice do elemento que */
         /* se encontra no meio da tabela  */
      meio = (inf + sup)/2;

         /* Ordena o elemento do meio e o primeiro elemento */
      if (tabela[meio] < tabela[inf])
         TrocaGenerica(tabela + inf, tabela + meio, sizeof(tabela[0]));

         /* Ordena o primeiro e o ltimo elementos */
      if (tabela[sup] < tabela[inf])
         TrocaGenerica(tabela + inf, tabela + sup, sizeof(tabela[0]));

         /* Ordena o elemento do meio e o ltimo elemento */
      if (tabela[sup] < tabela[meio])
         TrocaGenerica(tabela + meio, tabela + sup, sizeof(tabela[0]));

         /* O piv ser a mediana dos trs elementos ordenados */
      pivo = tabela[meio];

         /* Troca o piv de posio com o ltimo elemento */
      TrocaGenerica(tabela + meio, tabela + sup - 1, sizeof(tabela[0]));

         /* Efetua a partio */
      for (i = inf, j = sup - 1; ; ) {
            /* Enquantos os elementos da tabela forem menores */
            /* do que o piv, avana o ndice inferior        */
         while (tabela[++i] < pivo)
            ;

            /* Enquantos os elementos da tabela forem maiores */
            /* do que o piv, recua o ndice superior         */
         while (tabela[--j] > pivo)
            ;

            /* Se os ndices se cruzarem, */
            /* a diviso foi concluda    */
         if (i >= j)
            break;

            /* Os elementos nos ndices i e j esto em parties */
            /* erradas e  preciso troc-los de lugar            */
         TrocaGenerica(tabela + i, tabela + j, sizeof(tabela[0]));
      }

         /* Coloca o piv em sua correta posio */
      TrocaGenerica(tabela + i, tabela + sup - 1, sizeof(tabela[0]));

         /* Ordena recursivamente a primeira partio da tabela */
      Quick2(tabela, inf, i - 1);

         /* Ordena recursivamente a segunda partio da tabela */
      Quick2(tabela, i + 1, sup);
   }
}

/****
 *
 * QuickSort2(): Ordena uma tabela usando quick sort. Esta funo
 *              simplesmente prov uma interface para a funo
 *              Quick2() que realmente faz o servio.
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser a ordenada
 *      nElem (entrada) - nmero de elementos na tabela
 *
 * Retorno: Nada
 *
 ****/
void QuickSort2(int tabela[], int nElem)
{
      /* Simplesmente chama Quick2() para fazer o servio */
   Quick2(tabela, 0, nElem - 1);
}

/*********************** QuickSort 3 **********************/

/****
 *
 * NAleatorio(): Retorna um nmero aleatrio entre os valores
 *               recebidos como parmetros
 *
 * Parmetros:
 *      m, n (entrada) - valores que definem o intervalo de
 *                       nmeros sorteados
 *
 * Retorno: O nmero aleatrio gerado
 *
 * Observaes:
 *      1.  indiferente se m <= n ou n <= m
 *      2. Quando chamada pela primeira vez, esta funo
 *         alimenta o gerador de nmeros aleatrios
 *
 ****/
static int NAleatorio(int m, int n)
{
   static int primeiraChamada = 1; /* Esta varivel checa se a  */
                                   /* funo est sendo chamada */
                                   /* pela primeira vez e deve  */
                                   /* ter durao fixa          */
   int        menor, maior;

      /* Se esta for a primeira chamada da funo, */
      /* alimenta o gerador de nmeros aleatrios  */
   if (primeiraChamada) {
         /* Alimenta o gerador de nmeros aleatrios */
      srand(time(NULL));

         /* A prxima chamada no ser mais a primeira */
      primeiraChamada = 0;
   }

      /* Leva em considerao o fato de n  */
      /* poder ser menor ou maior do que m */
   if (n < m) {
      menor = n;
      maior = m;
   } else {
      menor = m;
      maior = n;
   }

   return rand()%(maior - menor + 1) + menor;
}

/****
 *
 * Quick3(): Ordena uma tabela usando o algoritmo QuickSort com
 *           o piv escolhido aleatoriamente
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser ordenada
 *      inf (entrada) - limite inferior da tabela
 *      sup (entrada) - limite superior da tabela
 *
 * Retorno: Nada
 *
 * Observaes:
 *   1. Por simplicidade, i.e., com o objetivo didtico
 *      de demonstrar o mtodo esta funo  implementada
 *      recursivamente, mas evidentemente uma implementao
 *      no recursiva seria mais eficiente
 *   2. O piv  escolhido aleatoriamente
 *
 ****/
static void Quick3(int tabela[], int inf, int sup)
{
   int i, j, pivo;

   if (inf >= sup)
      return; /* A tabela j deve estar ordenada */

      /* Nesta implementao, o piv  escolhido ao acaso */
   pivo = NAleatorio(inf, sup);

      /* Troca o piv de lugar com o primeiro elemento */
   TrocaGenerica(tabela + pivo, tabela + inf, sizeof(tabela[0]));

      /* O piv agora  o primeiro elemento */
   pivo = inf;

      /* Cria as parties */
   for (i = inf, j = sup; i < j; ) {
         /* Enquanto os elementos da tabela forem menores */
         /* do que o piv, avana-se o ndice inferior    */
      while (tabela[i] <= tabela[pivo] && i < sup)
         ++i;

         /* Enquanto os elementos da tabela forem maiores */
         /* do que o piv, recua-se o ndice superior     */
      while (tabela[j] > tabela[pivo])
         --j;

         /* Os elementos nos ndices i e j esto em parties */
         /* erradas e  preciso troc-los de lugar            */
      if (i < j)
         TrocaGenerica(tabela + i, tabela + j, sizeof(tabela[0]));
   }

      /* Coloca o piv em sua correta posio */
   TrocaGenerica(tabela + pivo, tabela + j, sizeof(tabela[0]));

      /* Ordena recursivamente a primeira metade da tabela */
   Quick3(tabela, inf, j - 1);

      /* Ordena recursivamente a segunda metade da tabela */
   Quick3(tabela, j + 1, sup);
}

/****
 *
 * QuickSort3(): Ordena uma tabela usando quick sort. Esta funo
 *              simplesmente prov uma interface para a funo
 *              Quick3() que realmente faz o servio.
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser a ordenada
 *      nElem (entrada) - nmero de elementos na tabela
 *
 * Retorno: Nada
 *
 ****/
void QuickSort3(int tabela[], int nElem)
{
      /* Simplesmente chama Quick3() para fazer o servio */
   Quick3(tabela, 0, nElem - 1);
}

/************************ MergeSort ***********************/

/****
 *
 * Intercala(): Faz a intercalao de duas tabelas
 *
 * Parmetros:
 *      tabela (entrada/sada) - primeira tabela a ser
 *                               intercalada
 *      aux (entrada/sada) - segunda tabela a ser intercalada
 *      iniEsq (entrada) - incio da metade da esquerda
 *      iniDir (entrada) - incio da metade da direita
 *      finalDir (entrada) - final da metade da direita
 *
 * Retorno: Nada
 *
 ****/
static void Intercala( int tabela[], int aux[],
                       int iniEsq, int iniDir, int finalDir )
{
   int i, finalEsq, nElementos, posAux;

   finalEsq = iniDir - 1;
   posAux = iniEsq;
   nElementos = finalDir - iniEsq + 1;

      /* Intercala as duas metades e coloca */
      /* o resultado na tabela auxiliar     */
   while((iniEsq <= finalEsq) && (iniDir <= finalDir))
      if(tabela[iniEsq] <= tabela[iniDir])
         aux[posAux++] = tabela[iniEsq++];
      else
         aux[posAux++] = tabela[iniDir++];

      /* Copia o restante da primeira metade */
   while(iniEsq <= finalEsq)
      aux[posAux++] = tabela[iniEsq++];

      /* Copia o restante da segunda metade */
   while(iniDir <= finalDir)
      aux[posAux++] = tabela[iniDir++];

      /* Copia a tabela auxiliar de */
      /* volta na tabela original   */
   for(i = 0; i < nElementos; i++,finalDir--)
      tabela[finalDir] = aux[finalDir];
}

/****
 *
 * MSort(): Ordena uma tabela usando intercalao com uma
 *          tabela auxiliar
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela a ser ordenada
 *      aux (entrada/sada) - tabela auxiliar
 *      esquerda (entrada) - incio da metade da esquerda da tabela
 *      direita (entrada) - incio da metade da direita da tabela
 *
 * Retorno: Nada
 *
 ****/
static void MSort(int *tabela, int *aux, int esquerda, int direita)
{
   int centro;

   if(esquerda < direita) {
      centro = (esquerda + direita)/2;

         /* Ordena metade inferior da tabela */
      MSort(tabela, aux, esquerda, centro);
         /* Ordena metade superior da tabela */
      MSort(tabela, aux, centro + 1, direita);

         /* Intercala as duas metades da */
         /* tabela na tabela auxiliar    */
      Intercala(tabela, aux, esquerda, centro + 1, direita);
   }
}

/****
 *
 * MergeSort(): Ordena uma tabela usando ordenao por intercalao
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser a ordenada
 *      nElem (entrada) - nmero de elementos na tabela
 *
 * Retorno: Nada
 *
 ****/
void MergeSort(int tabela[], int nElem)
{
   int *aux;

      /* Aloca espao para a tabela auxiliar */
   aux = malloc(nElem*sizeof(int));
   ASSEGURA(aux, "Impossivel alocar tabela auxiliar");

      /* Chama MSort() para fazer o servio */
   MSort(tabela, aux, 0, nElem - 1);

      /* Libera o espao ocupado pela tabela auxiliar */
   free(aux);
}

/************************ HeapSort ************************/

#define FILHO_E(x) (2*(x) + 1)
#define FILHO_D(x) (2*(x) + 2)
#define PAI(x) (((x) - 1)/2)

typedef int *tHeap;

/****
 *
 * OrdenaHeap(): Restaura a propriedade de ordenao de um
 *               heap de mnimo aps uma operao de remoo
 *
 * Parmetros:
 *      heap (entrada e sada) - ponteiro para o heap
 *      indice (entrada) - ndice do elemento do heap a partir
 *                         do qual ser efetuada a restaurao
 *      Compara (entrada) - ponteiro para uma funo que compara
 *                          dois elementos de array armazenados
 *                          no heap
 *
 * Retorno: Nada
 *
 ****/
static void OrdenaHeap( tHeap heap, int indice, int n )
{
   int iEsq = FILHO_E(indice), /* ndice do filho esquerdo */
       iDir = FILHO_D(indice), /* ndice do filho direito  */
       iMenor = indice; /* Supe que o maior elemento  o pai */

      /* Compara o filho esquerdo com seu pai */
   if ( iEsq < n && heap[iEsq] > heap[indice] )
         /* O filho esquerdo  maior do que o pai */
      iMenor = iEsq;

      /* Compara o filho direito com o   */
      /* maior entre seu pai e seu irmo */
   if ( iDir < n && heap[iDir] > heap[iMenor] )
      iMenor = iDir; /* O filho direito  o maior de todos */

      /* Se o n no for maior do que seus filhos, */
      /* troca-o de posio com o maior deles      */
   if (iMenor != indice) {
         /* O pai e o maior filho trocam de posio */
      TrocaGenerica(heap + indice, heap + iMenor, sizeof(int));

         /* Reordena o heap a partir do filho que se tornou pai */
      OrdenaHeap(heap, iMenor, n);
   }
}

/****
 *
 * HeapSort(): Ordena uma tabela usando ordenao por HeapSort
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser a ordenada
 *      nElem (entrada) - nmero de elementos na tabela
 *
 * Observaes:
 *       1. Como o heap  implementado numa rvore binria
 *          completa, as seguintes frmulas so vlidas para
 *          calcular o pai, o filho da esquerda e o filho da
 *          direita de um n numerado com i:
 *            - Pai(i) = (i-1)/2;
 *            - FilhoEsquerda(i) = 2*i+1; FilhoDireita(i) = 2*i+2
 *       2. Para ordenao em ordem crescente, o heap deve
 *          ser decrescente; caso contrrio, o heap deve
 *          ser crescente
 *
 * Retorno: Nada
 *
 ****/
void HeapSort(int tabela[], int nElem)
{
   int i;

      /* Constri o heap a partir do pai */
      /* do ltimo elemento da tabela    */
   for(i = (nElem - 2)/2; i >= 0; i--)
      OrdenaHeap(tabela, i, nElem);

      /* Neste ponto, o heap est construdo.   */
      /* Troca-se o primeiro elemento do heap,  */
      /* diminudo de um elemento, com o ltimo */
      /* elemento e reordena-se o heap.         */
   for(i = nElem-1; i >= 1; i--) {
         /* Remove o maior elemento do heap, coloca-o */
         /* no final da tabela e reordena o heap      */
      TrocaGenerica(tabela, tabela + i, sizeof(tabela[0]));
      OrdenaHeap(tabela, 0, i);
   }
}

/******************* Funes Auxiliares *******************/

/****
 *
 * TabelaEstaOrdenada(): Testa se uma tabela est ordenada em ordem crescente
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser a testada
 *      nElementos (entrada) - nmero de elementos na tabela
 *
 * Retorno: 1, se a tabela estiver ordenada;
 *          0, em caso contrrio
 *
 ****/
int TabelaEstaOrdenada(int tabela[], int nElementos)
{
   int i;

   for (i = 0; i < nElementos - 1; ++i)
      if (tabela[i] > tabela[i + 1])
         return 0; /* Encontrado um par fora de ordem */

      /* Se ainda no houve retorno, a tabela est ordenada */
   return 1;
}

/****
 *
 * TabelaEstaEmOrdemInversa(): Testa se uma tabela est ordenada em
 *                       ordem decrescente
 *
 * Parmetros:
 *      tabela (entrada/sada) - tabela que ser a testada
 *      nElem (entrada) - nmero de elementos na tabela
 *
 * Retorno: 1, se a tabela estiver ordenada em ordem decrescente;
 *          0, em caso contrrio
 *
 ****/
int TabelaEstaEmOrdemInversa(int tabela[], int nElementos)
{
   int i;

   for (i = 0; i < nElementos - 1; ++i)
      if (tabela[i] < tabela[i + 1])
         return 0; /* Encontrado um par fora de ordem */

      /* Se ainda no houve retorno, a tabela */
      /* est em ordem decrescente            */
   return 1;
}

/****
 *
 * InverteTabela(): Ordena uma tabela em ordem inversa uasando
 *                  qsort()
 *
 * Parmetros:
 *      tabela (entrada) - tabela que ser ordenada
 *      nChaves (entrada) - nmero de elementos da tabela
 *      compara (entrada) - ponteiro para uma fuino de comparao
 *
 * Retorno: Nada
 *
 ****/
void InverteTabela( int *tabela, int nChaves,
                    int (*compara)(const void *, const void*) )
{
   qsort(tabela, nChaves, sizeof(*tabela), compara);
}
