/****
 *
 * Ttulo: Ordenao de Listas Encadeadas
 *
 * Seo 11.8.1
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2017
 * ltima modificao: 16/04/2017
 *
 * Descrio: Exemplo de ordenao de listas encadeadas
 *
 * Entrada: Nada
 *
 * Sada: V. Exemplo de Execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>  /* E/S                        */
#include <stdlib.h> /* Alocao dinmica e rand() */
#include <time.h>   /* Funo time()              */
#include <string.h> /* Funo memmove()           */

/****************** Definies de Macros ******************/

   /* Macro utilizada para verificar     */
   /* ocorrncia de condies de exceo */
#define ASSEGURA( x, msg ) if (!(x) ) {\
                              printf("\n%s", msg); \
                              exit(1); \
                           }

/****************** Definies de Tipos *******************/

typedef struct rotNoLSE {
           unsigned         conteudo;
           struct rotNoLSE *proximo;
        } tNoListaSE, *tListaSE;

/************************* Aluses ************************/

extern void OrdenaListaSE(tNoListaSE *lista);
extern tListaSE CriaListaSE(int n, int m);
extern void ExibeListaSE(tListaSE lista);

/***************** Definies de Funes ******************/

/****
 *
 * TrocaGenerica(): Troca os valores de duas variveis do mesmo tamanho
 *
 * Parmetros:
 *      var1, var2 (entrada e sada) - endereos das variveis
 *                                     que sero trocadas
 *      tam (entrada) - tamanho das variveis que sero trocadas
 *
 * Retorno: 0, se no ocorrer erro; 1, em caso contrrio
 *
 ****/
static void TrocaGenerica(void *var1, void *var2, size_t tam)
{
   void *p;

   p = malloc(tam);
   ASSEGURA(p, "Impossivel trocar duas variaveis");

   memmove(p, var1, tam);
   memmove(var1, var2, tam);
   memmove(var2, p, tam);

   free(p);
}

/****
 *
 * OrdenaListaSE(): Ordena uma lista encadeada
 *
 * Parmetros:
 *    lista (entrada/sada) - a lista encadeada que ser ordenada
 *
 * Retorno: Nada
 *
 ****/
void OrdenaListaSE(tNoListaSE *lista)
{
   tNoListaSE *p, *q; /* Ponteiros usados para visitar os ns da lista */
   int         ordenada = 0; /* Informar se a lista est ordenada */

      /* Se a lista estiver vazia ou tiver apenas um elemento, ela est ordenada */
   if (!lista || !lista->proximo)
      return;

      /* O lao encerra quando no houver troca de ns */
   while (!ordenada) {
      ordenada = 1; /* Supe que a lista est ordenada */

      p = lista->proximo; /* p segue  frente de q */
      q = lista; /* q segue atrs de p */

         /* O lao a seguir encerra quando cada n  */
         /* tiver sido comparado com seu antecessor */
      while (p) {
            /* Compara contedos dos ns apontados por q e p */
         if (q->conteudo > p->conteudo) {
            ordenada = 0; /* Ns esto fora de ordem*/

               /* Troca os contedos dos ns */
            TrocaGenerica(&q->conteudo, &p->conteudo, sizeof(p->conteudo));
         }

            /* Passa para os ns seguintes */
         q = p;
         p = p->proximo;
      }
   }
}

/****
 *
 * CriaListaSE(): Cria uma lista encadeada com contedos aleatrios
 *
 * Parmetros:
 *      n (entrada) - nmero de ns da lista
 *      m (entrada) - maior valor de contedo
 *
 * Retorno: Endereo da lista criada
 *
 ****/
tListaSE CriaListaSE(int n, int m)
{
   tListaSE lista = NULL, p;
   int      i;

   srand(time(NULL));

   for (i = 0; i < n; ++i) {
      p = malloc(sizeof(int));

      p->conteudo = rand()%m;
      p->proximo = lista;
      lista = p;
   }

   return lista;
}

/****
 *
 * ExibeListaSE(): Exibe na tela o contedo do tipo int de cada n
 *               de uma lista encadeada
 *
 * Parmetros:
 *      lista (entrada) - a lista a ser exibida
 *
 * Retorno: Nada
 *
 ****/
void ExibeListaSE(tListaSE lista)
{
   while (lista) {
       printf("%d\t", lista->conteudo);
       lista = lista->proximo;
   }

   putchar('\n');
}

/****
 *
 * main(): Testa a funo OrdenaListaSE()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   tListaSE lista;

   lista = CriaListaSE(10, 20);

   printf("\n\t\t\t>>> Antes da ordenacao <<<\n\n");

   ExibeListaSE(lista);

   OrdenaListaSE(lista);

   printf("\n\t\t\t>>> Depois da ordenacao <<<\n\n");

   ExibeListaSE(lista);

   return 0;
}

/******************* Exemplo de Execuo ******************

                        >>> Antes da ordenacao <<<

4       17      15      19      14      17      6       17      2       8

                        >>> Depois da ordenacao <<<

2       4       6       8       14      15      17      17      17      19

**********************************************************/

