/****
 *
 * Ttulo: O Problema da Bandeira Holandesa
 *
 * Seo 11.8.3
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/05/2017
 * ltima modificao: 16/05/2017
 *
 * Descrio: Divide um array contendo apenas elementos
 *            com valores 0, 1 e 2 em  trs sees: a primeira
 *            contm elementos iguais a 0, a segunda contm iguais
 *            a 1 e a terceira contm elementos iguais a 2
 *
 * Entrada: Nada
 *
 * Sada: V. Exemplo de Execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/****************** Definies de Macros ******************/

   /* Macro utilizada para verificar     */
   /* ocorrncia de condies de exceo */
#define ASSEGURA( x, msg ) if (!(x) ) {\
                              printf("\n%s", msg); \
                              exit(1); \
                           }

/************************* Aluses ************************/

extern void SeparaEm3Particoes(int lista[], int n);
extern void ExibeArrayDeInts(int lista[], int n);

/***************** Definies de Funes ******************/

/****
 *
 * TrocaGenerica(): Troca os valores de duas variveis do mesmo tamanho
 *
 * Parmetros:
 *      var1, var2 (entrada e sada) - endereos das variveis
 *                                     que sero trocadas
 *      tam (entrada) - tamanho das variveis que sero trocadas
 *
 * Retorno: 0, se no ocorrer erro; 1, em caso contrrio
 *
 ****/
static void TrocaGenerica(void *var1, void *var2, size_t tam)
{
   void *p;

   p = malloc(tam);
   ASSEGURA(p, "Impossivel trocar duas variaveis");

   memmove(p, var1, tam);
   memmove(var1, var2, tam);
   memmove(var2, p, tam);

   free(p);
}

/****
 *
 * SeparaEm3Particoes(): Divide um array em trs parties: uma contendo
 *           valores iguais a 0, outra com valores iguais a
 *           1 e a ltima com valores iguais a 2
 *
 * Parmetros:
 *      lista (entrada) - a lista
 *      n (entrada) - nmero de elementos da lista
 *
 * Retorno: Nada
 *
 * Observao: Os nicos valores permitidos no array so 0, 1 e 2
 *
 ****/
void SeparaEm3Particoes(int lista[], int n)
{
   int inf = 0,
       sup = n - 1,
       meio = 0;

      /* Coloca cada elemento em sua devida partio */
   while (meio <= sup) {
      switch (lista[meio]) {
         case 0:
            TrocaGenerica(lista + inf, lista + meio, sizeof(lista[0]));
            ++inf;
            ++meio;
            break;
         case 1:
            meio++;
            break;
         case 2:
            TrocaGenerica(lista + meio, lista + sup, sizeof(lista[0]));
            --sup;
            break;
         default:
            printf("\nValor de array diferente de 0, 1 ou 2\n");
            exit(1);
      }
   }
}

/****
 *
 * ExibeArrayDeInts(): Exibe na tela o contedo do tipo int de cada elemento
 *               de um array
 *
 * Parmetros:
 *      lista (entrada) - a lista v
 *
 * Retorno: Nada
 *
 ****/
void ExibeArrayDeInts(int lista[], int n)
{
   int i;

   printf("\n{ ");

   for (i = 0; i < n - 1; ++i)
      printf("%d, ", lista[i]);

   printf("%d }\n", lista[i]);
}

/****
 *
 * main(): Testa a funo OrdenaLista()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int ar[] = {0, 1, 1, 0, 1, 2, 1, 2, 0, 0, 0, 1},
       tam = sizeof(ar)/sizeof(ar[0]);

   printf("\n>>> Lista original <<<\n");

   ExibeArrayDeInts(ar, tam);

   printf("\n>>> Lista separada <<<\n");

   SeparaEm3Particoes(ar, tam);

   ExibeArrayDeInts(ar, tam);

   return 0;
}

/******************* Exemplo de Execuo ******************

>>> Lista original <<<

{ 0, 1, 1, 0, 1, 2, 1, 2, 0, 0, 0, 1 }

>>> Lista separada <<<

{ 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 2, 2 }

**********************************************************/

