/****
 *
 * Ttulo: Encontrando Chaves Duplicatas Eficientemente
 *
 * Seo 11.8.4
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/05/2017
 * ltima modificao: 16/05/2017
 *
 * Descrio: Verifica se um array possui chaves duplicadas
 *            usando dois mtodos diferentes
 *
 * Entrada: Nada
 *
 * Sada: V. Exemplo de Execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>
#include <stdlib.h>

/************************* Aluses ************************/

extern int ContemDuplicatasListaIdx(int lista[], int n);
extern int ContemDuplicatasListaIdxOrd(int lista[], int n);
extern void ExibeArrayDeInts(int lista[], int n);

/***************** Definies de Funes ******************/

/****
 *
 * ContemDuplicatasListaIdx(): Verifica se uma lista indexada contm pelo
 *                     menos um elemento duplicado
 *
 * Parmetros:
 *      lista (entrada) - a lista
 *      n (entrada) - nmero de elementos
 *
 * Retorno: O ndice do primeiro valor duplicado encontrado, se
 *          ele existir; -1, em caso contrrio
 *
 ****/
int ContemDuplicatasListaIdx(int lista[], int n)
{
   int i, j;

      /* Compara elementos */
   for (i = 0; i < n; ++i)
      for (j = i + 1; j < n; ++j)
         if (lista[i] == lista[j])
            return i; /* Encontrada uma chave duplicada */

   return -1; /* No foi encontrada nenhuma duplicata */
}

/****
 *
 * ContemDuplicatasListaIdxOrd(): Verifica se uma lista indexada ordenada
 *                        contm pelo menos um elemento duplicado
 *
 * Parmetros:
 *      lista (entrada) - a lista
 *      n (entrada) - nmero de elementos
 *
 * Retorno: O ndice do primeiro valor duplicado encontrado, se
 *          ele existir; -1, em caso contrrio
 *
 * Observao: Esta funo supe que a lista est ordenada
 *
 ****/
int ContemDuplicatasListaIdxOrd(int lista[], int n)
{
   int i;

     /* Compara elementos adjacentes */
   for (i = 0; i < n - 1; ++i)
      if (lista[i] == lista[i + 1])
         return i; /* Encontrada uma chave duplicada */

   return -1; /* No foi encontrada nenhuma duplicata */
}

/****
 *
 * ExibeArrayDeInts(): Exibe na tela o contedo do tipo int de cada elemento
 *               de um array
 *
 * Parmetros:
 *      lista (entrada) - a lista
 *
 * Retorno: Nada
 *
 ****/
void ExibeArrayDeInts(int lista[], int n)
{
   int i;

   printf("\n{ ");

   for (i = 0; i < n - 1; ++i)
      printf("%d, ", lista[i]);

   printf("%d }\n", lista[i]);
}

int ComparaInts(const void *e1, const void *e2)
{
   return *(int *)e1 - *(int *)e2;
}

/****
 *
 * main(): Testa a funo OrdenaLista()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int ar[] = {2, -1, 8, -2, 5, -1, 3, 2},
       tam = sizeof(ar)/sizeof(ar[0]),
       teste;

   printf("\n>>> Lista original <<<\n");

   ExibeArrayDeInts(ar, tam);

   teste = ContemDuplicatasListaIdx(ar, tam);

   if (teste >= 0)
      printf("\nO elemento %d e' duplicado\n", ar[teste]);
   else
      printf("\nNao ha' elemento duplicado\n");

   printf("\n>>> Lista ordenada <<<\n");

   qsort(ar, tam, sizeof(ar[0]), ComparaInts);

   ExibeArrayDeInts(ar, tam);

   teste = ContemDuplicatasListaIdxOrd(ar, tam);

   if (teste >= 0)
      printf("\nO elemento %d e' duplicado\n", ar[teste]);
   else
      printf("\nNao ha' elemento duplicado\n");

   return 0;
}

/******************* Exemplo de Execuo ******************

>>> Lista original <<<

{ 2, -1, 8, -2, 5, -1, 3, 2 }

O elemento 2 e' duplicado

>>> Lista ordenada <<<

{ -2, -1, -1, 2, 2, 3, 5, 8 }

O elemento -1 e' duplicado

**********************************************************/

