/****
 *
 * Ttulo: Lista Bitnica
 *
 * Seo 11.8.5
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/05/2017
 * ltima modificao: 21/05/2018
 *
 * Descrio: Encontra o ponto bitnico de uma lista bitnica
 *
 * Entrada: Nada
 *
 * Sada: V. Exemplo de Execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>
#include <stdlib.h>

/************************* Aluses ************************/

extern int PontoBitonico(int lista[], int n);
extern void SomaDefinidaOrd(int lista[], int n, int soma);
extern void ExibeLista(int lista[], int n);

/***************** Definies de Funes ******************/

/****
 *
 * PontoBitonico(): Encontra o ponto bitnico de uma lista bitnica
 *
 * Parmetros:
 *      lista (entrada) - a lista
 *      n (entrada) - nmero de elementos da lista
 *
 * Retorno: O ndice do ponto bitnico, se ele existir;
 *          -1, em caso contrrio
 *
 ****/
int PontoBitonico(int lista[], int n)
{
   int inf = 0,
       sup = n - 1,
       meio;

   while (inf < sup) {
         /* Se o array s possui um elemento,  ele  o ponto bitnico */
      if (inf == sup)
         return inf;

         /* Se o array possui dois elementos, o ponto bitnico  o maior deles */
      if (inf == sup - 1)
         return lista[inf] > lista[sup] ? inf : sup;

      /*                                          */
      /* O array possui pelo menos trs elementos */
      /*                                          */

      meio = inf + (sup - inf)/2;

      if ( lista[meio - 1] < lista[meio] &&
           lista[meio] > lista[meio + 1] )
         return meio;
      else if ( lista[meio - 1] < lista[meio] &&
                lista[meio] < lista[meio + 1] )
         inf = meio + 1;
      else if ( lista[meio - 1] > lista[meio] &&
                lista[meio] > lista[meio + 1] )
         sup = meio - 1;
      else
         return -1; /* A lista no  bitnica */
   }

   return -1; /* A lista no  bitnica */
}

/****
 *
 * ExibeArrayDeInts(): Exibe na tela o contedo do tipo int de cada elemento
 *               de um array
 *
 * Parmetros:
 *      lista (entrada) - a lista
 *
 * Retorno: Nada
 *
 ****/
void ExibeArrayDeInts(int lista[], int n)
{
   int i;

   printf("\n{ ");

   for (i = 0; i < n - 1; ++i)
      printf("%d, ", lista[i]);

   printf("%d }\n", lista[i]);
}

/****
 *
 * main(): Testa a funo PontoBitonico()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
//   int ar[] = {5, 8, 9, 12, 6, 4, 1, 0},
   int ar[] = {1, 3, 8, 11, 21, 23, 54, 88},
       tam = sizeof(ar)/sizeof(ar[0]),
       pBitonico;

   printf("\n>>> Lista original <<<\n");

   ExibeArrayDeInts(ar, tam);

   pBitonico = PontoBitonico(ar, tam);

   if (pBitonico < 0)
      printf("\n\t>>> Nao ha' ponto bitonico\n");
   else
      printf("\n\t>>> Ponto bitonico = %d\n", ar[pBitonico]);

   return 0;
}

/******************* Exemplo de Execuo ******************

>>> Lista original <<<

{ 1, 3, 8, 11, 21, 23, 54, 88 }

        >>> Ponto bitonico = 88

**********************************************************/

