/****
 *
 * Ttulo: Testa ordenao externa de arquivos por intercalao
 *         binria
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/09/2017
 * ltima modificao: 15/05/2018
 *
 * Descrio: Testa ordenao externa de arquivos por intercalao
 *            binria
 *
 * Entrada:
 *
 * Sada:
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>     /* Funes de Entrada/Sada */
#include <string.h>    /* Processamento de strings */
#include <stdlib.h>    /* Funes de alocao dinmica */
#include <time.h>      /* Funo time() para medida de tempo */

/****************** Constantes Simblicas *****************/

   /* Nome do arquivo de dados original */
#define NOME_ARQUIVO_BIN "../CensoMEC2012.bin"
//#define NOME_ARQUIVO_BIN "MiniCensoMEC2012.bin"

   /* Nome do arquivo de dados ordenado */
#define NOME_ARQUIVO_ORD "CensoMEC2012_Ord.bin"

   /* Nmero mximo de registros em cada srie */
#define MAX_REGISTROS 8000

#define MAX_NOME_IES     200 /* Tamanho mximo de */
                             /* um nome de IES    */
#define MAX_NOME_CURSO   200 /* Tamanho mximo de */
                             /* um nome de curso  */

#define MAX_UF_NASCIMENTO 30 /* Tamanho mximo   */
                             /* de um nome de UF */

/****************** Definies de Macros ******************/

   /* Macro utilizada para verificar     */
   /* ocorrncia de condies de exceo */
#define ASSEGURA(condicao, msg) if (!(condicao)) {\
                                   fprintf(stderr, "\n%s\n", msg);\
                                   exit(1); \
                                }

/****************** Definies de Tipos *******************/

   /* Tipo da chave */
typedef long tChave;

   /* Tipo usado para armazenamento de */
   /* registros no arquivo binrio     */
typedef struct {
           long codigoAluno;
           long chave; /* Chave */
           char nomeIES[MAX_NOME_IES];
           char nomeCurso[MAX_NOME_CURSO];
           char sexoAluno;
           int  idadeAluno;
           char UFNascimento[MAX_UF_NASCIMENTO];
           int  anoIngresso;
         } tRegistro;

/***************** Definies de Funes ******************/

/****
 *
 * MedidaDeTempo(): Mede intervalos de tempo entre duas
 *                  chamadas consecutivas
 *
 * Parmetros: Nenhum
 *
 * Retorno: Nada
 *
 * Observao: Esta funo deve ser chamada aos pares; i.e.,
 *             na primeira chamada ela marca o incio da contagem
 *             de tempo e na segunda chamada ela apresenta o
 *             intervalo de tempo decorrido desde a ltima chamada
 *
 ****/
void MedidaDeTempo(void)
{
   static int    numeroDeChamadas = 0;
   static time_t t0; /* Instante inicial */
   time_t        tf; /* Instante final   */
   double        intervalo;

      /* Esta funo foi chamada mais uma vez */
   ++numeroDeChamadas;

      /* Se o nmero da chamada for mpar, esta */
      /* chamada  a primeira de um par         */
   if (numeroDeChamadas%2) { /* Calcula o instante inicial */
      t0 = time(NULL);
   } else { /* Calcula e apresenta o intervalo de   */
            /* tempo entre as duas ltimas chamadas */
      tf = time(NULL);

      intervalo = difftime(tf, t0);

      if (intervalo < 60) {
         printf( "\n\n\t*** Tempo gasto na operacao: "
                 "%5.2f segundos ***\n", intervalo );
      } else {
         printf( "\n\n\t*** Tempo gasto na operacao: "
                 "%5.2f minutos ***\n", intervalo/60 );
      }
   }
}

/****
 *
 * TestaOrdenacao(): Verifica se um arquivo est ordenado em
 *                   ordem crescente
 *
 * Parmetros:
 *      stream (entrada) - stream associado ao arquivo
 *
 * Retorno: 1, se o arquivo estiver ordenado; 0, em caso contrrio
 *
 ****/
int TestaOrdenacao(FILE *stream)
{
   tRegistro registroAtual, registroAnterior;
   int       teste;

   rewind(stream);

      /* L o primeiro registro do arquivo */
   fread(&registroAnterior, sizeof(registroAnterior), 1, stream);
   ASSEGURA( !ferror(stream), "Erro de leitura" );

      /* L os demais registros e testa se eles esto ordenados */
   while (1) {
         /* L o prximo registro */
      fread(&registroAtual, sizeof(registroAtual), 1, stream);
      ASSEGURA( !ferror(stream), "Erro de leitura" );

      if (feof(stream))
         break; /* Leitura acabou */

         /* Compara a chave do registro atual */
         /* com a chave do registro anterior  */
      teste = registroAtual.chave - registroAnterior.chave;

         /* Verifica se as duas ltimas chaves */
         /* esto em ordem crescente           */
      if (teste < 0)
         return 0;

         /* A chave anterior passa a ser a chave corrente */
      registroAnterior = registroAtual;
   }

      /* Se ainda no houve retorno, o arquivo est ordenado */
   return 1;
}

/****
 *
 * main(): Verifica se um arquivo est ordenado
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main (void)
{
   FILE *stream;
   int   resultado, nRegistros;

      /* Tenta abrir o arquivo de dados desordenado */
   stream = fopen(NOME_ARQUIVO_BIN, "rb");
   ASSEGURA(stream, "Arquivo desordenado nao foi aberto");

   fseek(stream, 0L, SEEK_END);

   nRegistros = ftell(stream) / sizeof(tRegistro);

   MedidaDeTempo();
   printf( "\nTestando ordenacao de %d registros do arquivo %s...",
           nRegistros, NOME_ARQUIVO_BIN );

   resultado = TestaOrdenacao(stream);

      /* Fecha o arquivo desordenado */
   fclose(stream);

   MedidaDeTempo();

   if (resultado)
      printf("\n\t>>> Ordenacao OK\n\n");
   else
      printf( "\a\n\t>>> O arquivo \"%s\" nao esta ordenado\n\n",
              NOME_ARQUIVO_BIN );

      /* Tenta abrir o arquivo de dados ordenado */
   stream = fopen(NOME_ARQUIVO_ORD, "rb");
   ASSEGURA(stream, "Arquivo ordenado nao foi aberto");

   fseek(stream, 0L, SEEK_END);

   nRegistros = ftell(stream) / sizeof(tRegistro);

   MedidaDeTempo();
   printf( "\nTestando ordenacao de %d registros do arquivo %s...",
           nRegistros, NOME_ARQUIVO_ORD );
   fflush(stdout);

   resultado = TestaOrdenacao(stream);

      /* Fecha o arquivo ordenado */
   fclose(stream);

   MedidaDeTempo();

   if (resultado)
      printf("\n\t>>> Ordenacao OK\n\n");
   else
      printf( "\a\n\t>>> O arquivo \"%s\" nao esta ordenado\n\n",
              NOME_ARQUIVO_ORD );

   return 0;
}

/******************* Exemplo de Execuo ******************

Testando ordenacao do arquivo J:\Dados\CensoMEC\CensoMEC2012.bin...

        *** Tempo gasto na operacao:  0.00 segundos ***

>>> O arquivo "J:\Dados\CensoMEC\CensoMEC2012.bin" nao esta ordenado


Testando ordenacao do arquivo CensoMEC2012_Ord.bin...

        *** Tempo gasto na operacao:  3.00 segundos ***

>>> Ordenacao OK

**********************************************************/
