/****
 *
 * Arquivo Tipos.h
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 07/06/2016
 * ltima modificao: 15/05/2018
 *
 * Descrio: Tipos utilizados pelo programa de ordenao de
 *            arquivos
 *
 ****/

#ifndef _Tipos_H_ /* Previne incluso mltipla deste arquivo */
#define _Tipos_H_

/*********************** Includes *************************/

#include "Macros.h" /* Macros usadas pelo programa */

/****************** Definies de Tipos *******************/

/* >>>>>>>>>>>>>>>>>>>>> Registros <<<<<<<<<<<<<<<<<<<<< */

   /* Tipo da chave */
typedef long tChave;

   /* Tipo usado para armazenamento de */
   /* registros no arquivo binrio     */
typedef struct {
           long codigoAluno;
           long chave; /* Chave */
           char nomeIES[MAX_NOME_IES];
           char nomeCurso[MAX_NOME_CURSO];
           char sexoAluno;
           int  idadeAluno;
           char UFNascimento[MAX_UF_NASCIMENTO];
           int  anoIngresso;
         } tRegistroMEC;

   /* Tipo do contedo armazenado em cada folha da rvore */
typedef struct  {
           tChave chave;  /* Chave do registro              */
           int    indice; /* Posio do registro no arquivo */
        } tChaveIndice;

/* >>>>>>>>>>>>>>>>>>>>> Ordenao <<<<<<<<<<<<<<<<<<<<< */

   /* Tipo de varivel que contm informao sobre uma srie */
typedef struct {
           long          pos; /* Posio da prxima leitura */
           tChaveIndice *buffer; /* Buffer associado  srie */
           int           nPares; /* Nmero de registros que */
                                 /* se encontram no buffer  */
           int           resto; /* Nmero de registros da srie */
                                /* que restam em arquivo        */
        } tSerie;

   /* Tipo de n de um heap */
typedef struct {
           tChaveIndice *elemento; /* Ponteiro para um par */
                                   /* de uma srie         */
           int           ir; /* ndice do par em seu buffer */
           int           is; /* ndice da srie  qual o par pertence */
        } tNoHeapIM_Arq2;

   /* Tipo de um heap */
typedef struct {
           tNoHeapIM_Arq2 *itens;  /* Array de elementos do heap   */
           int             nItens; /* Nmero de elementos do array */
        } tHeapIM_Arq2;

   /* Tipo de ponteiro para funo de comparao que compara registros */
typedef int (*tFCompara) (const void *, const void *);

/* >>>>>>>>>>>>>>>>>>>>> rvore B+ <<<<<<<<<<<<<<<<<<<<< */

   /* Constantes usadas para informar se   */
   /* um n  interno, folha ou est vazio */
typedef enum {FOLHA, INTERNO, NO_VAZIO} tTipoDoNo;

   /* Tipo usado para armazenamento de ns internos */
typedef struct {
           int    nFilhos;       /* Nmero de filhos do n */
           tChave chaves[G - 1]; /* Array de chaves        */
           int    filhos[G];     /* Array de filhos do n  */
        } tNoInterno;

   /* Tipo usado para armazenamento de folhas */
typedef struct {
           int          nChaves;    /* Nmero de chaves do n */
           tChaveIndice chaves[F];  /* Array de pares chave/ndice */
           int          proximaFolha; /* ndice da prxima folha */
        } tNoFolha;

   /* Tipo usado para armazenamento de   */
   /* ns (internos/folhas) da rvore B+ */
typedef struct {
           tTipoDoNo     tipoDoNo; /* Tipo do n */
           union {
              tNoInterno noInterno; /* Usado se o n  interno */
              tNoFolha   noFolha;   /* Usado se o n  folha   */
           } conteudo;
        } tNoBM, *tArvoreBM;

   /* Tipo usado para armazenamento do */
   /* caminho de um n at outro n    */
typedef struct {
           tNoBM no;       /* O n              */
           int   endereco; /* Seu endereo      */
           int   pos;      /* Sua posio entre seus irmos */
        } tNoCaminho;

typedef tNoCaminho tElemento; /* Apenas por convenincia */

   /* Tipo da pilha usada para guardar caminhos */
typedef struct noPilha {
           tElemento       dado;
           struct noPilha *proximo;
        } tNoPilha, *tPilha;

   /* Tipo de varivel usada para armazenar um n e sua posio em arquivo */
typedef struct {
           tNoBM  no;  /* O n */
           int    pos; /* Posio do n no arquivo que o contm */
        } tNoPos;

   /* Tipo de lista indexada usada para guardar    */
   /* ns ativos durante o processo de bulkloading */
typedef struct {
           tNoPos *nosPos; /* Array contendo ns ativos e suas */
                           /* respectivas posies em arquivo  */
           int     nNos;   /* Nmero de ns ativos */
           int     tamArray; /* Tamanho corrente do array */
        } tListaNosAtivos;

#endif
