/****
 *
 * Ttulo: Ordenao de Arquivos por Indexao
 *
 * Seo 12.7.1
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/05/2014
 * ltima modificao: 16/05/2014
 *
 * Descrio: Ordenao de arquivos por indexao
 *
 * Entrada: Arquivo binrio "Tudor.bin"
 *
 * Sada: V. Exemplo de Execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/***************** Constantes Simblicas ******************/

#define MAX_NOME      20 /* Nmero mximo de caracteres num nome */

#define TAM_MATR      4 /* Nmero de caracteres numa matrcula  */

#define MAX_REGISTROS 20 /* Nmero mximo de elementos na tabela */

   /* Nome do arquivo binrio (diretrio corrente) */
#define NOME_ARQ_BIN "Tudor.bin"

/****************** Definies de Macros ******************/

   /* Macro utilizada para verificar     */
   /* ocorrncia de condies de exceo */
#define ASSEGURA( x, msg ) if (!(x) ) {\
                              printf("\n%s", msg); \
                              exit(1); \
                           }

/****************** Definies de Tipos *******************/

   /* Tipo do nome */
typedef char tNome[MAX_NOME + 1];

   /* Tipo da matrcula */
typedef char tMatricula[TAM_MATR + 1];

typedef struct {
           tNome      nome;   /* Nome do aluno */
           tMatricula matr;   /* Sua matrcula */
           double     n1, n2; /* Suas notas    */
        } tAluno;

typedef  struct {
            int  indice;
            char chave[MAX_NOME + 1];
         } tParChaveIndice;

/************************* Aluses ************************/

extern int ExibeArquivoBin(FILE *stream);
extern int ColetaPares(FILE *stream, tParChaveIndice pares[], int maxPares);
extern int ComparaStr(const void *p1, const void *p2);
extern int ExibeRegistrosOrdenados( FILE *stream, const tParChaveIndice pares[],
                                    int nElementos );

/***************** Definies de Funes ******************/

/****
 *
 * MoveApontador(): Tenta mover o apontador de posio de um
 *                  arquivo e, quando isso no  possvel,
 *                  aborta o programa
 *
 * Parmetros:
 *      stream (entrada) - stream associado ao arquivo
 *      bytes (entrada) - quantidade de bytes que sero saltados
 *      DeOnde (entrada) - a partir de onde ocorrer a movimentao
 *
 * Retorno: Nada
 *
 ****/
void MoveApontador(FILE *stream, long bytes, int DeOnde)
{
   int deuErrado;

   deuErrado = fseek(stream, (long) bytes, DeOnde);

      /* Verifica se houve erro */
   if (deuErrado) {
      fprintf(stderr, "\n>>> Erro na tentativa de mover apontador de arquivo");
      exit(1);
   }
}

/****
 *
 * ExibeArquivoBin(): Exibe na tela os registros de um arquivo
 *                    binrio contendo estruturas do tipo tAluno
 *                    acessando-os sequencialmente
 *
 * Parmetros: stream (entrada) - stream associado ao arquivo
 *                                que ser exibido
 *
 * Retorno: Zero, se no houver erro.
 *          Um valor diferente de zero, em caso contrrio.
 *
 * Observao: O arquivo associado ao stream recebido como
 *             parmetro deve ter sido aberto em modo binrio
 *             que permite leitura
 *
 ****/
int ExibeArquivoBin(FILE *stream)
{
   tAluno registro; /* Armazena cada registro lido */

      /* Move indicador de posio para o incio do arquivo */
   MoveApontador(stream, 0L, SEEK_SET);

   /*                                                     */
   /* Antes de apresentar o cabealho, deve-se tentar ler */
   /* o primeiro registro, pois se isso no for feito e o */
   /* arquivo estiver vazio ou ocorrer erro de leitura,   */
   /* o cabealho aparecer sem dados abaixo dele.        */
   /*                                                     */

      /* Tenta ler o primeiro registro */
   fread(&registro, sizeof(tAluno), 1, stream);

      /* Se o final do arquivo foi atingido ou */
      /* ocorreu erro de leitura, retorna 1    */
   if (feof(stream) || ferror(stream))
      return 1; /* Nada foi apresentado */

      /* Escreve o cabealho da tabela */
   printf( "\n\tNome\t\tMatricula\tNota 1\tNota 2"
           "\n\t====\t\t=========\t======\t======\n\n" );

      /* Escreve o primeiro registro lido */
   printf( "    %-15s\t%6s\t\t %3.1f\t %3.1f\n",
           registro.nome, registro.matr, registro.n1,
           registro.n2 );

      /* Acessa cada registro do arquivo na ordem em   */
      /* que ele se encontra. O lao encerra quando o  */
      /* final do arquivo for atingido ou ocorrer erro */
   while (1) {
         /* L um registro */
      fread(&registro, sizeof(tAluno), 1, stream);

         /* Se o final do arquivo foi atingido ou   */
         /* ocorreu erro de leitura, encerra o lao */
      if (feof(stream) || ferror(stream)) {
         break;
      }

         /* Escreve o registro lido. No tente entender  */
         /* o embelezamento usado com printf(). Ele foi  */
         /* obtido por tentativa e erro, e no se aplica */
         /* a dados com outros formatos.                 */
      printf( "    %-15s\t%6s\t\t %3.1f\t %3.1f\n",
              registro.nome, registro.matr, registro.n1,
              registro.n2 );
   }

      /* Se no ocorreu erro de leitura, o retorno  0; */
      /* caso contrrio, o retorno  diferente de 0     */
   return ferror(stream);
}

/****
 *
 * ColetaPares(): L registros de um arquivo e armazena a chave
 *                de cada registro juntamente com seu ndice
 *
 * Parmetros:
 *     stream (entrada): stream onde ser feita a leitura
 *     pares (sada): array que armazenar pares chave/ndice
 *     maxRegistros (entrada): nmero mximo de registro lidos
 *
 * Retorno: O nmero de registros lidos, se no houver erro;
 *          zero, em caso contrrio.
 *
 ****/
int ColetaPares(FILE *stream, tParChaveIndice pares[], int maxPares)
{
   int    i;
   tAluno registro;

   rewind(stream); /* Move o indicador de posio para o incio do arquivo */

      /* L cada registro do arquivo e armazena sua chave e seu ndice no arquivo */
   for (i = 0; i < maxPares; i++) {
         /* L um registro */
      fread(&registro, sizeof(registro), 1, stream);

         /* Se o final de arquivo for atingido ou ocorrer */
         /* algum erro de leitura, encerra o lao         */
      if ( feof(stream) || ferror(stream) )
         break;

          /* Armazena o par chave/ndice do registro */
          /* lido num elemento do array pares[]      */
      strcpy(pares[i].chave, registro.nome);
      pares[i].indice = i;
   }

   return ferror(stream) ? -1 : i;
}

/****
 *
 * ComparaStr(): Funo de comparao utilizada por qsort()
 *
 * Parmetros:
 *      p1 (entrada): ponteiro para um par do tipo tParChaveIndice
 *      p2 (entrada): ponteiro para outro par do tipo tParChaveIndice
 *
 * Retorno: 0, se as chaves so iguais
 *          > 0, se a primeira chave  maior do que a segunda
 *          < 0, se a primeira chave  menor do que a segunda
 *
 * Observao: Utiliza a funo strcmp() para comparar os campos
 *             'nome' dos registros
 *
 ****/
int ComparaStr(const void *p1, const void *p2)
{
   return strcmp(((tParChaveIndice *)p1)->chave, ((tParChaveIndice *)p2)->chave);
}

/****
 *
 * OrdenaPares(): Classifica um array de elementos do
 *                tipo tParChaveIndice usando a funo qsort()
 *
 * Parmetros:
 *      pares (entrada/sada): array que armazena pares
 *                             chave/ndice
 *      nElementos (entrada): nmero elementos do array
 *
 * Retorno: Nada
 *
 ****/
void OrdenaPares( tParChaveIndice  pares[], int nElementos)
{
   qsort(pares, nElementos, sizeof(tParChaveIndice), ComparaStr);
}

/****
 *
 * ExibeRegistrosOrdenados(): Imprime registros ordenados
 *
 * Parmetros:
 *      stream (entrada) - stream onde ser feita a leitura
 *      pares (entrada) - array que armazena os pares
 *                       chave/ndice ordenados pela chave
 *      nElementos (entrada) - nmero de elementos do array
 *
 * Retorno: O nmero de registros lidos, se no houver erro;
 *          zero, em caso contrrio
 *
 ****/
int ExibeRegistrosOrdenados( FILE *stream, const tParChaveIndice pares[],
                             int nElementos )
{
   tAluno registro;
   int    i;

      /* Verifica se h elementos a serem escritos */
   if (!nElementos)
      return 1; /* No h */

      /* Escreve o cabealho da tabela */
   printf( "\n\tNome\t\tMatricula\tNota 1\tNota 2"
           "\n\t====\t\t=========\t======\t======\n\n" );

      /* Acessa cada registro do arquivo na ordem   */
      /* indicada pela sua posio no array pares[] */
   for (i = 0;  i < nElementos; i++) {
         /* Move o indicador de posio do arquivo para  */
         /* o local calculado utilizando pares[i].indice */
      MoveApontador(stream, sizeof(tAluno)*pares[i].indice, SEEK_SET);

      fread(&registro, sizeof(tAluno), 1, stream); /* L o registro */

         /* Se o final do arquivo foi atingido ou   */
         /* ocorreu erro de leitura, encerra o lao */
      if (feof(stream) || ferror(stream))
         break;

         /* Escreve o registro lido com algum embelezamento */
      printf( "    %-15s\t%6s\t\t %3.1f\t %3.1f\n", registro.nome,
              registro.matr, registro.n1, registro.n2 );
   }

      /* Se no ocorreu erro de leitura, o retorno  0; */
      /* caso contrrio, o retorno  diferente de 0     */
   return ferror(stream);
}

/****
 *
 * main(): Testa ordenao de arquivos por indexao
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   FILE   *stream;
   tParChaveIndice  pares[MAX_REGISTROS];
   int     nRegistrosLidos;

      /* Abre arquivo em modo binrio para leitura apenas */
   stream = fopen(NOME_ARQ_BIN, "rb");

      /* Se o arquivo no pode ser aberto encerra o programa */
   if (!stream) {
      printf( "Impossivel abrir arquivo \"%s\"\n",
              NOME_ARQ_BIN );
      return 1; /* Arquivo no foi aberto */
   }
            /*                                  */
            /* >>> Exibe arquivo como ele  <<< */
            /*                                  */

   printf("\n\t\t>>> Arquivo Original <<<\n");

      /* Se ocorrer erro durante a exibio */
      /* do arquivo, encerra o programa     */
   if ( ExibeArquivoBin(stream) ) {
      printf("Erro processando arquivo.\n");
      fclose(stream); /* Fecha arquivo antes de partir */
      return 1; /* Ocorreu erro */
   }

            /* >>> Exibe arquivo ordenado <<< */

   printf("\n\t\t>>> Arquivo Ordenado <<<\n");

      /* Obtm o array de pares ndice/chave */
   nRegistrosLidos = ColetaPares( stream, pares, MAX_REGISTROS );

   if (nRegistrosLidos > 0) { /* Coleta de pares foi OK */
         /* Ordena o array de pares ndice/chave */
      OrdenaPares(pares, nRegistrosLidos);

         /* Tenta exibir os registro ordenados */
         /* verificando se ocorreu algum erro  */
      if (ExibeRegistrosOrdenados(stream, pares, nRegistrosLidos)){
         printf("Erro processando arquivo.\n");
         fclose(stream); /* Fecha arquivo antes de partir */
         return 1; /* Ocorreu erro na funo   */
                   /* ExibeRegistrosOrdenados()*/
      }
   } else { /* Erro na coleta de pares ndice/chave */
      printf("Erro processando arquivo.\n");
      fclose(stream); /* Fecha arquivo antes de partir */
      return 1; /* Ocorreu erro na funo ColetaPares() */
   }

   fclose(stream); /* Fecha arquivo */

   return 0;
}

/******************* Exemplo de Execuo ******************

                >>> Arquivo Original <<<

        Nome            Matricula       Nota 1  Nota 2
        ====            =========       ======  ======

    Catarina Parr         1440           4.0     6.0
    Catarina Howard       1340           6.0     7.7
    Henrique VIII         1029           9.5     9.0
    Ana de Cleves         1022           4.5     6.0
    Joana Seymour         1017           7.7     8.7
    Catarina Aragon       1014           5.5     6.5
    Ana Bolena            1012           7.8     8.0

                >>> Arquivo Ordenado <<<

        Nome            Matricula       Nota 1  Nota 2
        ====            =========       ======  ======

    Ana Bolena            1012           7.8     8.0
    Ana de Cleves         1022           4.5     6.0
    Catarina Aragon       1014           5.5     6.5
    Catarina Howard       1340           6.0     7.7
    Catarina Parr         1440           4.0     6.0
    Henrique VIII         1029           9.5     9.0
    Joana Seymour         1017           7.7     8.7

**********************************************************/
